Option Strict Off
Option Explicit On
Friend Class clsDeltaT
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Calculates DeltaT - The time between UTC1 and TD

    Function DeltaT(ByVal dblFixed As Double, ByVal blnValid As Boolean, Optional ByVal blnFromUTC As Boolean = True) As Double
        'Find the difference between UTC1 and TD
        'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 77
        'according to
        'J. Chapront, M. Chapront-Touze and G. Francou, Note s055, bureau des Longitudes, Paris, Dec. 1997
        'dblfixed is fixed radix from 1 jan 1 0:00 UTC gregorian = 1
        Dim dblJYear As Double 'Julian year - th formulae are in Julian years
        Dim intI As Short 'Counter
        Dim dblY(3) As Double 'Used for interpolation of deltat
        Dim dblT As Double 'Julian centuries from 2000
        Dim dblCorrection As Double 'Correction for continuity
        Dim blnFinished As Boolean 'Flag for successful completion of iteration of reverse conversion of DeltaT
        Dim dblOldDeltaT As Double 'DeltaT from previous iteration
        Dim strLines() As String 'Used for reading data files
        Dim intLines As Integer 'Number of lines read from data file
        Dim intLineNumber As Integer 'Current line of data file
        Dim cReadResource As New clsReadResource


        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFixed) > FixMax And blnFromUTC Or System.Math.Abs(dblFixed) > FixMax + 30 And Not blnFromUTC Then
            'if converting to UTC then give a little more leaway and check at the end
            Err.Raise(vbObjectError + 2, "clsDeltaT.DeltaT", "Magnitude of dblFixed too large: " & dblFixed)
        End If
        If Not blnDeltaTRead Then 'First time, read table (biannual 1620 to 2004)
            ReDim strLines(0)
            intLines = 0
            Call cReadResource.ReadResource(My.Resources.DeltaT, strLines, intLines, "_", "'", True, ",")
            intLineNumber = -1
            For intI = 0 To NDeltaTModern - 1
                intLineNumber = intLineNumber + 1
                If intLineNumber >= intLines Then
                    Err.Raise(vbObjectError + 1, "clsDeltaT.DeltaT", "Error in DeltaT file: File truncated at entry: " & intLineNumber - 1)
                End If
                If Not Main.GetValue(strLines(intLineNumber), "Double", -10, 130) Then
                    Err.Raise(vbObjectError + 1, "clsDeltaT.DeltaT", "Invalid number in file DeltaT.txt, entry " & intI & ": " & strLines(intLineNumber))
                End If
                dblT = intI / 50 + 16.2
                dblDeltaT(intI) = strLines(intLineNumber)
                'If preatomic time then adjust for lunar acceleration
                If dblT < 19.555 Then dblDeltaT(intI) = dblDeltaT(intI) - 0.91072 * (LunAcc - LunAccDTTables) * Square(dblT - 19.555)
                'LunAccDTTables - Lunar acceleration in "/cy^2 assumed by Morrison et al. in Delta T tables prior to 1955.5
                'LunAcc - current measured value of lunar acceleration (25.858 "/cy^2)
            Next
            ReDim strLines(0)
            intLines = 0
            intLineNumber = -1
            Call cReadResource.ReadResource(My.Resources.PastDeltaT, strLines, intLines, "_", "'", True, ",")
            For intI = 0 To 42 'Old Delta T values from -500 to 1600 derived from eclipse tables
                'F. R. Stephenson, Historical Eclipses and Earth's Rotation, Cambridge University Press (1997)
                intLineNumber = intLineNumber + 1
                If intLineNumber >= intLines Then
                    Err.Raise(vbObjectError + 1, "clsDeltaT.DeltaT", "Error in PastDeltaT file: File truncated at entry: " & intLineNumber - 1)
                End If
                If Not Main.GetValue(strLines(intLineNumber), "Double", 100, 17000) Then
                    Err.Raise(vbObjectError + 1, "clsDeltaT.DeltaT", "Invalid number in file PastDeltaT.txt, entry " & intI & ": " & strLines(intLineNumber))
                End If
                dblT = intI / 2 - 5
                dblPastDeltaT(intI) = strLines(intLineNumber)
                'Adjust for lunar acceleration
                dblPastDeltaT(intI) = dblPastDeltaT(intI) - 0.91072 * (LunAcc - LunAccPast) * Square(dblT - 19.555)
                'LunAccPast - Lunar acceleration assumed by Stephenson
            Next
            Erase strLines 'Free up memory
            blnDeltaTRead = True
        End If
        DeltaT = 0
        dblOldDeltaT = 0
        blnFinished = False
        While Not blnFinished
            dblJYear = (dblFixed + 14.25 - DeltaT / 86400 + 365.25) / 365.25 'Julian year
            dblT = (dblJYear - 2000) / 100 'Julian centuries from 2000
            If dblJYear < -500 Or dblJYear > 1618 + 2 * NDeltaTModern Then 'Distant past use JPL formula and correct for difference in lunar acceleration
                'F. R. Stephenson & M. A. Houlden, Atlas of Historical Eclipse Maps: East Asia 1500 BC - AD 1700, Cambridge University Press (1986)
                'DeltaT = 31 * Square((dblJYear - 1820) / 100) - 0.91072 * (LunAcc - LunAccPast) * Square((dblJYear - 1955.5) / 100)
                'Extrapolation assuming the LunAcc parameter for lunar accleration
                DeltaT = (dblJYear / 100 - 18.6108)
                DeltaT = 31.3097 * Square(DeltaT) - 326.8 + 370 * System.Math.Sin(0.40251 * (DeltaT - 11.48)) - 39.7 * System.Math.Sin(0.71644 * (DeltaT - 13.98))
                If dblJYear > -600 And dblJYear < 0 Then 'Interpolate to the beginning of the first table
                    'Smoothing to extrapolation
                    DeltaT = DeltaT - (DeltaT - dblPastDeltaT(0)) * (dblJYear + 600) / 100
                ElseIf dblJYear < 1718 + 2 * NDeltaTModern And dblJYear > 0 Then  'Smooth away to future extrapolation
                    DeltaT = DeltaT - (DeltaT - dblDeltaT(NDeltaTModern - 1)) * (1718 + 2 * NDeltaTModern - dblJYear) / 100
                End If
            ElseIf dblJYear < -475 Then  'Linear interpolation of 1st two points of first table
                dblY(0) = dblPastDeltaT(0)
                dblY(1) = dblPastDeltaT(1)
                DeltaT = Interp(dblY, (dblJYear + 500) / 50, 2, True)
            ElseIf dblJYear < 1575 Then  'Three point interpolation of first table
                dblY(0) = dblPastDeltaT((Int(dblJYear) + 475) \ 50)
                dblY(1) = dblPastDeltaT((Int(dblJYear) + 475) \ 50 + 1)
                dblY(2) = dblPastDeltaT((Int(dblJYear) + 475) \ 50 + 2)
                DeltaT = Interp(dblY, dblJYear / 50 - Int((dblJYear - 25) / 50), 3, True)
            ElseIf dblJYear < 1600 Then  'Linear interpolation of last two points
                dblY(0) = dblPastDeltaT(41)
                dblY(1) = dblPastDeltaT(42)
                DeltaT = Interp(dblY, (dblJYear - 1550) / 50, 2, True)
            ElseIf dblJYear < 1620 Then  'No data between 1600 and 1620 so linear interpolation
                dblY(0) = dblPastDeltaT(42)
                dblY(1) = dblDeltaT(0)
                DeltaT = Interp(dblY, (dblJYear - 1600) / 20, 2, True)
            ElseIf dblJYear < 1621 Then  'Linear interpolation between first two points of second table
                dblY(0) = dblDeltaT(0)
                dblY(1) = dblDeltaT(1)
                DeltaT = Interp(dblY, (dblJYear - 1620) / 2, 2, True)
            ElseIf dblJYear > 1617 + NDeltaTModern * 2 Then  'Linear interpolation between last two points of last table
                dblY(0) = dblDeltaT(190)
                dblY(1) = dblDeltaT(191)
                DeltaT = Interp(dblY, (dblJYear - 1616 - NDeltaTModern * 2) / 2, 2, True)
            Else 'Three point interpolation of second table
                dblY(0) = dblDeltaT((Int(dblJYear) - 1621) \ 2)
                dblY(1) = dblDeltaT((Int(dblJYear) - 1621) \ 2 + 1)
                dblY(2) = dblDeltaT((Int(dblJYear) - 1621) \ 2 + 2)
                DeltaT = Interp(dblY, dblJYear / 2 - Int((dblJYear - 1) / 2), 3, True)
            End If
            blnFinished = blnFromUTC Or System.Math.Abs(DeltaT - dblOldDeltaT) < 100
            'if converting to UTC and date in ephemeris time and year very large an iteration is required to maintain consistency
            dblOldDeltaT = DeltaT
        End While
        If System.Math.Abs(dblFixed - DeltaT / 86400) > FixMax And Not blnFromUTC Then
            'if converting to UTC recheck at the end that it is not out of range
            Err.Raise(vbObjectError + 2, "clsDeltaT.DeltaT", "Magnitude of dblFixed too large: " & dblFixed)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            FileClose(1)
            End
        ElseIf Err.Number = vbObjectError + 2 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            FileClose(1)
            blnValid = False
        ElseIf Err.Number = 53 Then
            MsgBox("Error in clsDeltaT.DeltaT " & Err.Description & vbCrLf & "File containing deltat data missing.")
            FileClose(1)
            End
        Else
            Err.Raise(Err.Number)
        End If
    End Function
End Class