Option Strict Off
Option Explicit On
Friend Class clsGregorian
	'(C)Roy E. Hoffman 2000
	'See Main.bas for terms and conditions of use
	'Most of the algorithms in this module are from
	'E. M. Reingold and N. Dershowitz "Calendrical Calculations: The Millenium Edition", Cambridge Univ. Press, Cambridge, UK, 1999.
	
    Const GregorianEpoch As Short = 1
    Const YearMin As Integer = -FixMax / 365.2425
    Const YearMax As Integer = FixMax / 365.2425
	
    Function GregorianLeapYear(ByVal intYear As Short) As Boolean
        'Is it a leap year
        GregorianLeapYear = (MyMod(intYear, 4) = 0 And (MyMod(intYear, 100) > 0 Or MyMod(intYear, 400) = 0))
    End Function
    Friend Function gregorian_fixed(ByVal udtDate As MyDate, ByRef blnValid As Boolean) As Integer
        'Returns fixed radix date
        Static lastDate As MyDate
        Static lastResult As Double
        Dim intY As Integer

        On Error GoTo EH
        blnValid = True
        If udtDate.enmDateType <> Main.DateType.gregorian Then
            Err.Raise(vbObjectError + 1, "cGregorian.gregorian_fixed", "Invalid enmDateType: " & udtDate.enmDateType)
        End If
        If udtDate.intYear > YearMax Or udtDate.intYear < YearMin And Not udtDate.blnCE Or -udtDate.intYear + 1 < YearMin And udtDate.blnCE Then
            Err.Raise(vbObjectError + 1, "cGregorian.gregorian_fixed", "Invalid intYear: " & udtDate.intYear)
        End If
        If udtDate.intMonth < 1 Or udtDate.intMonth > 12 Then
            Err.Raise(vbObjectError + 1, "cGregorian.gregorian_fixed", "Invalid intMonth: " & udtDate.intMonth)
        End If
        If udtDate.intDateOfMonth < 1 Or udtDate.intDateOfMonth > DaysInGregorianMonth(udtDate.intMonth, udtDate.intYear, blnValid) Then
            Err.Raise(vbObjectError + 1, "cGregorian.gregorian_fixed", "Invalid intMonth: " & udtDate.intMonth)
        End If
        If Not blnValid Then Exit Function
        If lastDate.intDateOfMonth = udtDate.intDateOfMonth And lastDate.intMonth = udtDate.intMonth And lastDate.intYear = udtDate.intYear Then
            gregorian_fixed = lastResult
        Else
            'Obtain year, removing BCE
            If udtDate.blnCE Then
                intY = udtDate.intYear
            Else
                intY = -udtDate.intYear + 1
            End If
            'Find approximate day number
            gregorian_fixed = GregorianEpoch - 1 + 365 * (intY - 1) + Int((intY - 1) / 4) - Int((intY - 1) / 100) + Int((intY - 1) / 400) + Int((367 * CInt(udtDate.intMonth) - 362) / 12) + udtDate.intDateOfMonth
            'Adjust in leap year
            If udtDate.intMonth > 2 And GregorianLeapYear(CShort(intY)) Then
                gregorian_fixed = gregorian_fixed - 1
            ElseIf udtDate.intMonth > 2 Then
                gregorian_fixed = gregorian_fixed - 2
            End If
            'Store last conversion so that the calculation is not repeated
            lastDate.blnCE = udtDate.blnCE
            lastDate.intDateOfMonth = udtDate.intDateOfMonth
            lastDate.intMonth = udtDate.intMonth
            lastDate.intYear = udtDate.intYear
            lastResult = gregorian_fixed
        End If
        If System.Math.Abs(gregorian_fixed) > FixMax Then
            Err.Raise(vbObjectError + 1, "cGregorian.gregorian_fixed", "Magnitude of gregorian_fixed too large: " & gregorian_fixed)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Function fixed_gregorianYear(ByVal lngDate As Integer, ByRef blnValid As Boolean) As Integer
        'Returns the Gregorian year
        Dim lngD0 As Integer 'Number of days in the Gregorian epoch
        Dim lngD1 As Integer 'Number of days in current 400 year cycle
        Dim lngD2 As Integer 'Number of days in current century
        Dim lngD3 As Integer 'Number of days in current 4 year cycle
        Dim intN400 As Short 'Number of 400 year cycles
        Dim intN100 As Short 'Number of centuries
        Dim intN4 As Short   'Number of 4 year cycles
        Dim intN1 As Short   'Number of years
        Static lastDate As Double
        Static lastResult As Integer

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(lngDate) > FixMax Then
            Err.Raise(vbObjectError + 1, "cGregorian.fixed_GregorianYear", "Magnitude of lngDate too large: " & lngDate)
        End If
        If lngDate <> 0 And lngDate = lastDate Then
            fixed_gregorianYear = lastResult
        Else
            lngD0 = lngDate - GregorianEpoch
            intN400 = Int(lngD0 / 146097)
            lngD1 = MyMod(lngD0, 146097)
            intN100 = Int(lngD1 / 36524)
            lngD2 = MyMod(lngD1, 36524)
            intN4 = Int(lngD2 / 1461)
            lngD3 = MyMod(lngD2, 1461)
            intN1 = Int(lngD3 / 365)
            fixed_gregorianYear = 400.0# * intN400 + 100 * intN100 + 4 * intN4 + intN1
            If intN1 < 4 And intN100 < 4 Then fixed_gregorianYear = fixed_gregorianYear + 1
            lastDate = lngDate
            lastResult = fixed_gregorianYear
            If System.Math.Abs(fixed_gregorianYear) > YearMax Then
                Err.Raise(vbObjectError + 1, "cGregorian.fixed_gregorianYear", "Magnitude of fixed_gregorianYear too large: " & fixed_gregorianYear)
            End If
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub fixed_gregorian(ByVal lngDate As Integer, ByRef udtDateOut As MyDate, ByRef blnValid As Boolean)
        'Convert a date number to a gregorian date
        Dim lngPriorDays As Integer
        Dim intCorrection As Short
        Dim lngYear As Integer
        Static lastDateOut As MyDate
        Static lastDate As Double

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(lngDate) > FixMax Then
            Err.Raise(vbObjectError + 1, "cGregorian.fixed_gregorian", "Magnitude of lngDate too large: " & lngDate)
        End If
        udtDateOut.enmDateType = Main.DateType.gregorian
        If lastDate = lngDate And lngDate <> 0 Then
            udtDateOut.intDateOfMonth = lastDateOut.intDateOfMonth
            udtDateOut.intMonth = lastDateOut.intMonth
            udtDateOut.intYear = lastDateOut.intYear
            udtDateOut.blnCE = lastDateOut.blnCE
        Else
            lngYear = fixed_gregorianYear(lngDate, blnValid) 'Find year
            If Not blnValid Then Exit Sub
            udtDateOut.blnCE = True 'Calculate the day number of start of year
            udtDateOut.intYear = lngYear
            udtDateOut.intMonth = 1
            udtDateOut.intDateOfMonth = 1
            lngPriorDays = lngDate - gregorian_fixed(udtDateOut, blnValid)
            If Not blnValid Then Exit Sub
            udtDateOut.intMonth = 3
            'If after February, adjust before calculating the date
            If lngDate >= gregorian_fixed(udtDateOut, blnValid) Then
                If GregorianLeapYear(udtDateOut.intYear) Then
                    intCorrection = 1
                Else
                    intCorrection = 2
                End If
            Else
                intCorrection = 0
            End If
            If Not blnValid Then Exit Sub
            udtDateOut.intMonth = Int((12 * (lngPriorDays + intCorrection) + 373) / 367)
            If udtDateOut.intMonth < 1 Or udtDateOut.intMonth > 12 Then
                Err.Raise(vbObjectError + 1, "cGregorian.fixed_gregorian", "Invalid udtDateOut.intMonth: " & udtDateOut.intMonth)
            End If
            udtDateOut.intDateOfMonth = Int(lngDate - gregorian_fixed(udtDateOut, blnValid) + 1)
            If Not blnValid Then Exit Sub
            If udtDateOut.intDateOfMonth < 1 Or udtDateOut.intDateOfMonth > DaysInGregorianMonth(udtDateOut.intMonth, udtDateOut.intYear, blnValid) Then
                Err.Raise(vbObjectError + 1, "cGregorian.fixed_gregorian", "Invalid udtDateOut.intDateOfMonth: " & udtDateOut.intDateOfMonth)
            End If
            If Not blnValid Then Exit Sub
            lastDate = lngDate
            lastDateOut.intDateOfMonth = udtDateOut.intDateOfMonth
            lastDateOut.intMonth = udtDateOut.intMonth
            lastDateOut.intYear = udtDateOut.intYear
            lastDateOut.blnCE = udtDateOut.blnCE
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function MonthName_Renamed(ByVal intMonth As Short) As String
        'Returns the name of the month (Gregorian or Julian)
        Static lastMonth As Short
        Static lastMonthName As String

        On Error GoTo EH
        If intMonth = lastMonth Then
            MonthName_Renamed = lastMonthName
        Else
            MonthName_Renamed = strTexts(intfrmDateBase + (56 + intMonth) * intNLanguages + intLanguage)
            '1 "January"
            '2 "February"
            '3 "March"
            '4 "April"
            '5 "May"
            '6 "June"
            '7 "July"
            '8 "August"
            '9 "September"
            '10 "October"
            '11 "November"
            '12 "December"
            lastMonth = intMonth
            lastMonthName = MonthName_Renamed
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Function DaysInGregorianMonth(ByVal intMonth As Short, ByVal intYear As Short, ByRef blnValid As Boolean) As Short
        'Last date of the month
        On Error GoTo EH
        blnValid = True
        If intMonth < 1 Or intMonth > 12 Then
            Err.Raise(vbObjectError + 1, "cGregorian.DaysInGregorianMonth", "Invalid intMonth: " & intMonth)
        End If
        If intMonth = 4 Or intMonth = 6 Or intMonth = 9 Or intMonth = 11 Then
            DaysInGregorianMonth = 30
        ElseIf intMonth = 2 Then
            If GregorianLeapYear(intYear) Then
                DaysInGregorianMonth = 29
            Else
                DaysInGregorianMonth = 28
            End If
        Else
            DaysInGregorianMonth = 31
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Function BeforeOnAfter(ByVal intDayOfWeek As Short, ByVal intDateOfMonth As Short, ByVal intWeekNo As Short, ByVal intMonth As Short, ByVal dblDate As Double, ByRef blnValid As Boolean, Optional ByVal blnAvoidFestivals As Boolean = False) As Short
        'Returns -1 before, 0 on, 1 after
        'intWeekNo is the number of weeks after the date on the intDayOfWeek defined
        'intWeekNo 0 means on that date and intWeekNo 5 means the last week of the month
        'Used for changes between summer and winter time
        Dim udtDateIn As MyDate
        Dim udtChangeover As MyDate
        Dim intDateNo2 As Short
        Dim intMonthNo2 As Short
        Dim cHebrew As New clsHebrew

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblDate) > FixMax Then
            Err.Raise(vbObjectError + 1, "cGregorian.BeforeOnAfter", "Magnitude of dblDate too large: " & dblDate)
        End If
        If intDayOfWeek < 0 Or intDayOfWeek > 6 Then
            Err.Raise(vbObjectError + 1, "cGregorian.BeforeOnAfter", "Invalid intDayOfWeek: " & intDayOfWeek)
        End If
        If intMonth < 1 Or intMonth > 12 Then
            Err.Raise(vbObjectError + 1, "cGregorian.BeforeOnAfter", "Invalid intMonth: " & intMonth)
        End If
        If (intDateOfMonth < 1 Or intDateOfMonth > 31) Or intDateOfMonth = 31 And (intMonth = 2 Or intMonth = 4 Or intMonth = 6 Or intMonth = 9 Or intMonth = 11) Or intDateOfMonth = 30 And intMonth = 2 Then
            Err.Raise(vbObjectError + 1, "cGregorian.BeforeOnAfter", "Invalid intDateOfMonth: " & intDateOfMonth)
        End If
        If intWeekNo < 0 Or intWeekNo > 5 Then
            Err.Raise(vbObjectError + 1, "cGregorian.BeforeOnAfter", "Invalid intWeekNo: " & intWeekNo)
        End If
        Call fixed_gregorian(Int(dblDate), udtDateIn, blnValid)
        Call fixed_gregorian(Int(dblDate), udtChangeover, blnValid)
        If Not blnValid Then Exit Function
        'Months
        If intMonth = udtDateIn.intMonth - 1 And intDateOfMonth > DaysInGregorianMonth(intMonth, udtDateIn.intYear, blnValid) - 6 Then
            If Not blnValid Then Exit Function
            udtChangeover.intDateOfMonth = intDateOfMonth - DaysInGregorianMonth(intMonth, Int(dblDate / 365.2425), blnValid)
            If Not blnValid Then Exit Function
            intDateNo2 = udtDateIn.intDateOfMonth
            intMonthNo2 = udtDateIn.intMonth
            If udtChangeover.intDateOfMonth > 0 Then
                udtDateIn.intDateOfMonth = udtChangeover.intDateOfMonth
            Else
                intDateOfMonth = udtChangeover.intDateOfMonth + DaysInGregorianMonth(intMonth, Int(dblDate / 365.2425), blnValid)
                udtDateIn.intMonth = udtDateIn.intMonth - 1
            End If
            udtChangeover.intDateOfMonth = udtChangeover.intDateOfMonth + MyMod(intDayOfWeek - fixed_DayOfWeek(gregorian_fixed(udtDateIn, blnValid)), 7)
            udtDateIn.intDateOfMonth = intDateNo2
            udtDateIn.intMonth = intMonthNo2
            If blnAvoidFestivals Then
                While cHebrew.IsFestival(gregorian_fixed(udtChangeover, blnValid))
                    udtChangeover.intDateOfMonth = udtChangeover.intDateOfMonth + 1
                    If udtChangeover.intDateOfMonth > DaysInGregorianMonth(udtChangeover.intMonth, udtChangeover.intYear, blnValid) Then
                        udtChangeover.intMonth = udtChangeover.intMonth + 1
                        udtChangeover.intDateOfMonth = 1
                        If udtChangeover.intMonth > 12 Then
                            udtChangeover.intMonth = 1
                            udtChangeover.intYear = udtChangeover.intYear + 1
                        End If
                    End If
                End While
            End If
            If udtDateIn.intDateOfMonth > udtChangeover.intDateOfMonth Then
                BeforeOnAfter = 1
            ElseIf udtDateIn.intDateOfMonth = udtChangeover.intDateOfMonth Then
                BeforeOnAfter = 0
            Else 'Before the date
                BeforeOnAfter = -1
            End If
        ElseIf intMonth > udtDateIn.intMonth Then
            BeforeOnAfter = -1
        ElseIf intMonth < udtDateIn.intMonth Then
            BeforeOnAfter = 1
        Else 'If same month
            If intWeekNo = 0 Then
                udtChangeover.intDateOfMonth = intDateOfMonth
            ElseIf intWeekNo = 5 Then  'Last week of month
                udtChangeover.intDateOfMonth = DaysInGregorianMonth(udtDateIn.intMonth, udtDateIn.intYear, blnValid) - 6
                If Not blnValid Then Exit Function
            Else 'Count the weeks from the date, 1 is 1st week
                udtChangeover.intDateOfMonth = intDateOfMonth + 7 * (intWeekNo - 1)
            End If
            If intWeekNo > 0 Then 'Select the date itself, not the week
                intDateNo2 = udtDateIn.intDateOfMonth
                udtDateIn.intDateOfMonth = udtChangeover.intDateOfMonth
                udtChangeover.intDateOfMonth = udtChangeover.intDateOfMonth + MyMod(intDayOfWeek - fixed_DayOfWeek(frmDate.EvaluateDate(udtDateIn, blnValid)), 7)
                If Not blnValid Then Exit Function
                udtDateIn.intDateOfMonth = intDateNo2
            End If
            If blnAvoidFestivals Then
                'Clock changes cannot occur on Jewish festivals as of 2011 in Israel.
                'Effectively this affects only 2nd Day Rosh Hashannah and the first instance
                'is in 2027.
                While cHebrew.IsFestival(gregorian_fixed(udtChangeover, blnValid))
                    'If it is a festival add one day
                    udtChangeover.intDateOfMonth = udtChangeover.intDateOfMonth + 1
                    If udtChangeover.intDateOfMonth > DaysInGregorianMonth(udtChangeover.intMonth, udtChangeover.intYear, blnValid) Then
                        udtChangeover.intMonth = udtChangeover.intMonth + 1
                        udtChangeover.intDateOfMonth = 1
                        If udtChangeover.intMonth > 12 Then
                            udtChangeover.intMonth = 1
                            udtChangeover.intYear = udtChangeover.intYear + 1
                        End If
                    End If
                End While
            End If
            If udtDateIn.intDateOfMonth > udtChangeover.intDateOfMonth Then
                BeforeOnAfter = 1
            ElseIf udtDateIn.intDateOfMonth = udtChangeover.intDateOfMonth Then
                BeforeOnAfter = 0
            Else 'Before the date
                BeforeOnAfter = -1
            End If
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Function fixed_DayOfWeek(ByVal dblDate As Double) As Short
        'Modulus of fixed radix day number
        '0 = Sunday
        fixed_DayOfWeek = MyMod(Int(dblDate), 7)
    End Function
End Class