Option Strict Off
Option Explicit On
Friend Class clsIslamic
    '(C)Roy E. Hoffman 2000-2011
    'Deals with the Islamic arithmetic calendar
	'See Main.bas for terms and conditions of use
	'Most of the algorithms in this module are from
	'E. M. Reingold and N. Dershowitz "Calendrical Calculations: The Millenium Edition", Cambridge Univ. Press, Cambridge, UK, 1999.
	
	Const IslamicEpoch As Integer = 227015
    Const YearMin As Integer = -(FixMax + 227015) / 354.3666667
    Const YearMax As Integer = (FixMax - 227015) / 354.3666667
    Function IslamicLeapYear(ByRef intYear As Short) As Boolean
        'Is it a leap year (is there a 30th Du Al Hijja)
        IslamicLeapYear = MyMod(intYear * 11.0# + 14, 30) < 11
    End Function
    Friend Function Islamic_Fixed(ByVal udtDate As MyDate, ByRef blnValid As Boolean) As Integer
        Dim intY As Integer 'Year number adjusted to remove 'BCE'
        'Convert an Islamic arithmetic date to fixed radix
        On Error GoTo EH
        blnValid = True
        If udtDate.intYear > YearMax Or udtDate.intYear < YearMin And Not udtDate.blnCE Or -udtDate.intYear + 1 < YearMin And udtDate.blnCE Then
            Err.Raise(vbObjectError + 1, "cIslamic.Islamic_fixed", "Magnitude of udtDate.intYear too large: " & udtDate.intYear)
        End If
        If udtDate.intMonth < 1 Or udtDate.intMonth > 12 Then
            Err.Raise(vbObjectError + 1, "cIslamic.Islamic_fixed", "Invalid udtDate.intMonth: " & udtDate.intMonth)
        End If
        If udtDate.intDateOfMonth < 1 Or udtDate.intDateOfMonth > 30 Then
            Err.Raise(vbObjectError + 1, "cIslamic.Islamic_fixed", "Invalid udtDate.intDateOfMonth: " & udtDate.intDateOfMonth)
        End If
        If udtDate.blnCE Then
            intY = udtDate.intYear
        Else
            intY = -udtDate.intYear + 1
        End If
        Islamic_Fixed = udtDate.intDateOfMonth + 29 * (udtDate.intMonth - 1) + Int((6 * udtDate.intMonth - 1) / 11) + CInt(udtDate.intYear - 1) * 354 + Int((3 + 11 * CInt(intY)) / 30) + IslamicEpoch - 1
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub Fixed_Islamic(ByVal lngDate As Integer, ByRef udtDateOut As MyDate, ByRef blnValid As Boolean)
        'Convert fixed radix to an Islamic arithmetic date
        Dim intPriorDays As Short 'Days since start of year
        Dim lngYear As Integer 'Year
        Static lastDate As Integer = 0 'Save previous value
        Static lastDateOut As MyDate

        On Error GoTo EH
        blnValid = True
        udtDateOut.enmDateType = Main.DateType.Islamic
        If lastDate = lngDate And lastDate <> 0 Then
            udtDateOut.intDateOfMonth = lastDateOut.intDateOfMonth
            udtDateOut.intMonth = lastDateOut.intMonth
            udtDateOut.intYear = lastDateOut.intYear
            udtDateOut.blnCE = lastDateOut.blnCE
        Else
            lngYear = Int((30 * (lngDate - IslamicEpoch) + 10646) / 10631) 'Determine the year
            If System.Math.Abs(lngYear) > 32765 Then
                Err.Raise(vbObjectError + 1, "cIslamic.Fixed_Islamic", "Magnitude of lngYear too large: " & lngYear)
            End If
            udtDateOut.intYear = lngYear
            udtDateOut.blnCE = True
            udtDateOut.intMonth = 1
            udtDateOut.intDateOfMonth = 1
            intPriorDays = Int(lngDate) - Islamic_Fixed(udtDateOut, blnValid)
            udtDateOut.intMonth = Int((11 * intPriorDays + 330) / 325)
            udtDateOut.intDateOfMonth = Int(lngDate) - Islamic_Fixed(udtDateOut, blnValid) + 1
            lastDate = lngDate
            lastDateOut.intDateOfMonth = udtDateOut.intDateOfMonth
            lastDateOut.intMonth = udtDateOut.intMonth
            lastDateOut.intYear = udtDateOut.intYear
            lastDateOut.blnCE = udtDateOut.blnCE
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function MonthName_Renamed(ByVal intMonth As Short) As String
        'Return the name of the Islamic month
        Static lastMonth As Short = 0 'Save previous value
        Static lastResult As String
        If lastMonth = intMonth And intMonth <> 0 Then
            MonthName_Renamed = lastResult
        Else
            MonthName_Renamed = strTexts(intfrmDateBase + (44 + intMonth) * intNLanguages + intLanguage)
            'Select Case intMonth
            '    Case 1
            'MonthName_Renamed = "Muharram"
            '    Case 2
            'MonthName_Renamed = "Safar"
            '    Case 3
            'MonthName_Renamed = "Rabi' I"
            '    Case 4
            'MonthName_Renamed = "Rabi' II"
            '    Case 5
            'MonthName_Renamed = "Jamada I"
            '    Case 6
            'MonthName_Renamed = "Jamada II"
            '    Case 7
            'MonthName_Renamed = "Rajab"
            '    Case 8
            'MonthName_Renamed = "Sha'ban"
            '    Case 9
            'MonthName_Renamed = "Ramadan"
            '    Case 10
            'MonthName_Renamed = "Shawwal"
            '    Case 11
            'MonthName_Renamed = "Dhu-al-Qu'da"
            '    Case 12
            'MonthName_Renamed = "Dhu-al-Hijja"
            lastMonth = intMonth
            lastResult = MonthName_Renamed
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
End Class