Option Strict Off
Option Explicit On
Friend Class clsJDE
    '(C)Roy E. Hoffman 2000-2011
    'Julian day ephimerides functions
	'See Main.bas for terms and conditions of use
	
    Friend Function jde_fixed(ByVal dblDate As Double, ByRef blnValid As Boolean) As Double
        'Convert Julian ephemirides day number (DT based) to fixed radix
        Dim cDeltaT As New clsDeltaT

        On Error GoTo EH
        blnValid = True
        'Adjust for deltaT
        jde_fixed = dblDate - 1721424.5 - cDeltaT.DeltaT(dblDate - 1721424.5, blnValid, False) / 86400
        If Not blnValid Then Exit Function
        If System.Math.Abs(jde_fixed) > FixMax Then
            Err.Raise(vbObjectError + 1, "clsJDE.jde_fixed", "Magnitude of jde_fixed too large: " & jde_fixed)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Function fixed_jde(ByVal dblDate As Double, ByRef blnValid As Boolean) As Double
        'Convert fixed radix to Julian ephemerides day number (DT based)
        Dim cDeltaT As New clsDeltaT

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblDate) > FixMax Then
            Err.Raise(vbObjectError + 1, "clsJDE.fixed_jde", "Magnitude of dblDate too large: " & dblDate)
        End If
        'Adjust for deltaT
        fixed_jde = dblDate + 1721424.5 + cDeltaT.DeltaT(dblDate, blnValid, True) / 86400
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
End Class