Option Strict Off
Option Explicit On
Friend Class clsJulian
    '(C)Roy E. Hoffman 2000-2011
    'Julian calendar functions
	'See Main.bas for terms and conditions of use
	'Most of the algorithms in this module are from
	'E. M. Reingold and N. Dershowitz "Calendrical Calculations: The Millenium Edition", Cambridge Univ. Press, Cambridge, UK, 1999.
	
    Const JulianEpoch As Short = -1
    Const YearMin As Integer = -(FixMax + 1) / 365.25
    Const YearMax As Integer = (FixMax - 1) / 365.25

	Function JulianLeapYear(ByRef intYear As Short, ByRef blnCE As Boolean) As Boolean
		'Is it a Julian leap year?
		JulianLeapYear = MyMod(intYear, 4) = 0 And blnCE Or MyMod(intYear, 4) = 1 And Not blnCE
	End Function
	Friend Function julian_fixed(ByRef udtDate As MyDate, ByRef blnValid As Boolean) As Integer
		'Convert Julian calendar to fixed radix
        Dim intY As Short 'Year adjusted to remove BCE
        Static lastDate As MyDate 'Save result
		Static lastResult As Double
		
		On Error GoTo EH
		blnValid = True
		If udtDate.intDateOfMonth = lastDate.intDateOfMonth And udtDate.intMonth = lastDate.intMonth And udtDate.intYear = lastDate.intYear And udtDate.blnCE = lastDate.blnCE Then
			julian_fixed = lastResult
		Else
            If udtDate.intYear > YearMax Or udtDate.intYear < YearMin And Not udtDate.blnCE Or -udtDate.intYear + 1 < YearMin And udtDate.blnCE Then
                Err.Raise(vbObjectError + 1, "cJulian.julian_fixed", "Magnitude of udtDate.intYear too large: " & udtDate.intYear)
            End If
			If udtDate.intMonth < 1 Or udtDate.intMonth > 12 Then
				Err.Raise(vbObjectError + 1, "cJulian.julian_fixed", "Invalid udtDate.intMonth: " & udtDate.intMonth)
			End If
			If udtDate.intDateOfMonth < 1 Or udtDate.intDateOfMonth > 31 Or udtDate.intDateOfMonth = 31 And (udtDate.intMonth = 2 Or udtDate.intMonth = 4 Or udtDate.intMonth = 6 Or udtDate.intMonth = 9 Or udtDate.intMonth = 11) Or udtDate.intMonth = 2 And (udtDate.intDateOfMonth = 30 Or udtDate.intDateOfMonth = 29 And Not JulianLeapYear(udtDate.intYear, udtDate.blnCE)) Then
				Err.Raise(vbObjectError + 1, "cJulian.julian_fixed", "Invalid udtDate.intDateOfMonth: " & udtDate.intDateOfMonth)
			End If
            If udtDate.blnCE Then 'Remove BCE factor
                intY = udtDate.intYear
            Else
                intY = -udtDate.intYear + 1
            End If
            julian_fixed = JulianEpoch - 1 + 365 * (CInt(intY) - 1) + Int((intY - 1) / 4) + Int((367.0# * udtDate.intMonth - 362) / 12) + udtDate.intDateOfMonth
            'Adjust for February
            If udtDate.intMonth > 2 And JulianLeapYear(udtDate.intYear, udtDate.blnCE) Then
                julian_fixed = julian_fixed - 1
            ElseIf udtDate.intMonth > 2 Then
                julian_fixed = julian_fixed - 2
            End If
			If System.Math.Abs(julian_fixed) > FixMax + 2 Then
				Err.Raise(vbObjectError + 1, "cJulian.julian_fixed", "Magnitude of julian_fixed too large: " & julian_fixed)
			End If
			lastDate.blnCE = udtDate.blnCE
			lastDate.intDateOfMonth = udtDate.intDateOfMonth
			lastDate.intMonth = udtDate.intMonth
			lastDate.intYear = udtDate.intYear
			lastResult = julian_fixed
		End If
		Exit Function
EH: 
		If Err.Number = vbObjectError + 1 Then
			MsgBox("Error in " & Err.Source & ", " & Err.Description)
			blnValid = False
		Else
			Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
			Err.Raise(Err.Number)
		End If
	End Function
	Friend Sub fixed_julian(ByRef lngDate As Integer, ByRef udtDateOut As MyDate, ByRef blnValid As Boolean)
		'Convert fixed radix to the Julian calendar
        Dim intY As Short 'Year adjusted to insert BCE
        Dim lngPriorDays As Integer 'Number of days since the start of the year
        Dim intCorrection As Short 'Correction in March for the length of February
        Dim lngYear As Integer 'Year
        Static lastDate As Integer = 0 'Save result
		Static lastDateOut As MyDate
		
		On Error GoTo EH
		blnValid = True
		If System.Math.Abs(lngDate) > FixMax + 2 Then
			Err.Raise(vbObjectError + 1, "cJulian.fixed_julian", "Magnitude of lngDate too large: " & lngDate)
		End If
		udtDateOut.enmDateType = Main.DateType.julian
		If lastDate = lngDate And dblDate <> 0 Then
			udtDateOut.intDateOfMonth = lastDateOut.intDateOfMonth
			udtDateOut.intMonth = lastDateOut.intMonth
			udtDateOut.intYear = lastDateOut.intYear
			udtDateOut.blnCE = lastDateOut.blnCE
		Else
            lngYear = Int((4 * (lngDate - JulianEpoch) + 1464) / 1461) 'Determine the year
			If System.Math.Abs(lngYear) > 32765 Then
				Err.Raise(vbObjectError + 1, "cJulian.fixed_julian", "Magnitude of lngYear too large: " & lngYear)
			End If
			intY = lngYear
			udtDateOut.blnCE = intY > 0
            If intY <= 0 Then 'Convert to BCE if before year 1
                udtDateOut.intYear = -intY + 1
            Else
                udtDateOut.intYear = intY
            End If
			udtDateOut.intMonth = 1
			udtDateOut.intDateOfMonth = 1
            lngPriorDays = lngDate - julian_fixed(udtDateOut, blnValid) 'Number of days since start of year
			If Not blnValid Then Exit Sub
			udtDateOut.intMonth = 3
            If lngDate >= julian_fixed(udtDateOut, blnValid) Then 'Correct for February
                If JulianLeapYear(udtDateOut.intYear, udtDateOut.blnCE) Then
                    intCorrection = 1
                Else
                    intCorrection = 2
                End If
            Else
                intCorrection = 0
            End If
            If Not blnValid Then Exit Sub
            'Find the month and date
			udtDateOut.intMonth = Int((12 * (lngPriorDays + intCorrection) + 373) / 367)
			udtDateOut.intDateOfMonth = Int(lngDate - julian_fixed(udtDateOut, blnValid) + 1)
			If Not blnValid Then Exit Sub
			lastDate = lngDate
			'udtDateOut.blnCE = True
			If intY < 0 Then
				udtDateOut.blnCE = False
			End If
			If udtDateOut.intMonth < 1 Or udtDateOut.intMonth > 12 Then
				Err.Raise(vbObjectError + 1, "cJulian.fixed_julian", "Invalid udtDateout.intMonth: " & udtDate.intMonth)
			End If
			If udtDateOut.intDateOfMonth < 1 Or udtDateOut.intDateOfMonth > 31 Or udtDateOut.intDateOfMonth = 31 And (udtDateOut.intMonth = 2 Or udtDateOut.intMonth = 4 Or udtDateOut.intMonth = 6 Or udtDateOut.intMonth = 9 Or udtDateOut.intMonth = 11) Or udtDate.intMonth = 2 And (udtDateOut.intDateOfMonth = 30 Or udtDateOut.intDateOfMonth = 29 And Not JulianLeapYear(udtDateOut.intYear, udtDateOut.blnCE)) Then
				Err.Raise(vbObjectError + 1, "cJulian.fixed_julian", "Invalid udtDateout.intDateOfMonth: " & udtDateOut.intDateOfMonth)
			End If
			lastDateOut.intDateOfMonth = udtDateOut.intDateOfMonth
			lastDateOut.intMonth = udtDateOut.intMonth
			lastDateOut.intYear = udtDateOut.intYear
			lastDateOut.blnCE = udtDateOut.blnCE
		End If
		Exit Sub
EH: 
		If Err.Number = vbObjectError + 1 Then
			MsgBox("Error in " & Err.Source & ", " & Err.Description)
			blnValid = False
		Else
			Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
			Err.Raise(Err.Number)
		End If
	End Sub
End Class