Option Strict Off
Option Explicit On
Friend Class clsLST
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Routines related to local sidreal time
	
	'Some of the algorithms from
	'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 87
    Friend Function FromLST(ByVal udtTime As Mytime, ByRef blnValid As Boolean, ByRef enmType As Main.DateType) As Double
        Dim cDMS As New clsDMS
        Dim dblT As Double 'Date in Julian centuries from J2000
        Dim dblT0 As Double 'Time in days
        Dim cHebrew As New clsHebrew

        On Error GoTo EH
        blnValid = True
        'Julian centuries from 2000
        dblT = (Int(dblDate - 2451545) + 1721424.5) / 36525 'Julian centuries at 0 UTC
        'Time at 0 UTC
        dblT0 = (6.697374558 + dblT * (2400.0513369072 + dblT * (0.000025862 - dblT * 0.00000000172))) / 24 + udtLocation.dblLongitude / 360 * 0.99726956633
        dblT0 = dblT0 - Int(dblT0)
        FromLST = cDMS.FromDMS(udtTime.udtDms, blnValid) / 24 - dblT0
        If Not blnValid Then Exit Function
        If udtTime.blnSecondTime Then FromLST = FromLST + 1
        FromLST = (FromLST - Int(FromLST)) * 0.99726956633
        If udtTime.blnSecondTime Then FromLST = FromLST + 0.9972695663
        If FromLST > 1 And enmType < Main.DateType.Hebrew Then
            blnValid = False
        ElseIf enmType >= Main.DateType.Hebrew Then
            Call cHebrew.AdjustTime(Int(dblDate), FromLST, blnValid, udtTime.blnSecondTime)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
	Friend Sub ToLST(ByRef dblFix As Double, ByRef udtTimeIn As Mytime, ByRef blnValid As Boolean)
		Dim cDMS As New clsDMS
        Dim cUTC As New clsUTC
        Dim dblT As Double 'Date in Julian centuries from J2000
        Dim dblT0 As Double 'Time in days
		Dim dblTime2 As Double
        Dim dblStart As Double
        Dim dblEnd As Double
        Dim intDayShift As Short

		On Error GoTo EH
		blnValid = True
		If System.Math.Abs(dblFix) > FixMax Then
			Err.Raise(vbObjectError + 1, "cLST.ToLST", "Magnitude of dblDate too large: " & dblFix)
		End If
		udtTimeIn.bln24Hr = True
		udtTimeIn.enmTimeType = Main.TimeType.lst
        dblT = (Int(dblFix - 2451545) + 1721424.5) / 36525
        dblT0 = (6.697374558 + dblT * (2400.0513369072 + dblT * (0.000025862 - dblT * 0.00000000172))) / 24 + udtLocation.dblLongitude / 360 * 0.99726956633
		dblT0 = dblT0 - Int(dblT0)
        Call cUTC.StartEndOfDay(dblFix, udtDate.enmDateType, dblStart, dblEnd, blnValid, intDayShift)
        If Not blnValid Then Exit Sub
        udtTimeIn.blnSecondTime = (dblFix - dblStart) / 0.99726956633 > 1
        dblTime2 = (dblFix - Int(dblFix)) / 0.99726956633 + dblT0
		dblTime2 = dblTime2 - Int(dblTime2)
		Call cDMS.ToDMS(dblTime2 * 24, udtTimeIn.udtDms)
		If udtTimeIn.udtDms.intDegreesHours > 23 Then
			Err.Raise(vbObjectError + 1, "cLST.ToLST", "Invalid udtTimeIn.udtDms.intDegreesHours: " & udtTimeIn.udtDms.intDegreesHours)
		End If
		Exit Sub
EH: 
		If Err.Number = vbObjectError + 1 Then
			MsgBox("Error in " & Err.Source & ", " & Err.Description)
			blnValid = False
		Else
			Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
			Err.Raise(Err.Number)
		End If
	End Sub
    Friend Function StartOfDay(ByVal dblDate As Double, ByVal udtTime As Mytime, ByVal udtLoc As Location) As Double
        'Find the time at the start of the day
        StartOfDay = -udtLoc.dblLongitude / 360
    End Function
    Friend Function FixToLST(ByVal dblFix As Double, ByVal blnTD As Boolean, ByRef blnValid As Boolean) As Double 'In degrees
        'dblFix is fixed radix in UTC or TD to LST in degrees
        Dim cGMST As New clsGMST

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax + 2 Then
            'if converting to UTC then give a little more leaway and check at the end
            Err.Raise(vbObjectError + 1, "cLST.FixToLST", "Magnitude of dblFix too large: " & dblFix)
        End If
        If udtLocation.dblLongitude <= -180 Or udtLocation.dblLongitude > 180 Then
            'if converting to UTC then give a little more leaway and check at the end
            Err.Raise(vbObjectError + 1, "cLST.FixToLST", "Invalid longitude: " & udtLocation.dblLongitude)
        End If
        FixToLST = FracCirc(cGMST.FixToGMST(dblFix, blnTD, blnValid) + udtLocation.dblLongitude)
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Err.Raise(Err.Number)
        End If
    End Function
End Class