Option Strict Off
Option Explicit On
Friend Class clsLocalTime
    '(C)Roy E. Hoffman 2000-2011
    'Deals with the local time zone
	'See Main.bas for terms and conditions of use
	
    Friend Function GetTZ(ByVal dblFix As Double, ByRef blnValid As Boolean, ByVal blnSecondTime As Boolean, ByRef blnChangeOverJump As Boolean) As Double
        'Find the time zone for the date in the country/state
        Dim intSpring As Short 'Date for change from winter to summer time
        Dim intFall As Short 'Date for change from summer to winter time
        Dim intI As Short 'Counter
        Dim cGregorian As New clsGregorian
        Dim cHebrew As New clsHebrew
        Dim cDMS As New clsDMS

        On Error GoTo EH
        blnValid = True
        blnChangeOverJump = False 'Flag if the time is within the hour skipped changing from winter to summer time
        If System.Math.Abs(dblFix) > FixMax + 2 Then
            Err.Raise(vbObjectError + 1, "cLocalTime.GetTZ", "Magnitude of dblDate too large: " & dblFix)
        End If
        If Not udtLocation.blnDSTAuto Then
            If udtLocation.blnDST Then 'Set according to manual DST setting
                GetTZ = udtLocation.dblTZ + 1
            Else
                GetTZ = udtLocation.dblTZ
            End If
        Else
            'Go through the list of DST regimes for that country
            intI = 0
            While udtLocation.udtTZ(intI).blnLast And udtLocation.udtTZ(intI).dblLast < dblFix And intI < udtLocation.intNTZ - 1
                intI = intI + 1
            End While 'Until the regime is found that is relevant to the date
            If Not udtLocation.udtTZ(intI).blnDST Then 'There is no DST at this location and era
                GetTZ = udtLocation.udtTZ(intI).dblTZ
            Else 'Determine if the date is before or after the time changes for this year
                With udtLocation.udtTZ(intI).udtDSTStart
                    If .enmDateType = Main.DateType.gregorian Then
                        intSpring = cGregorian.BeforeOnAfter(.intDayOfWeek, .intDate, .intWeekNo, .intMonthNo, dblFix, blnValid, udtLocation.udtTZ(intI).blnAvoidFestivals)
                        If Not blnValid Then Exit Function
                    ElseIf .enmDateType = Main.DateType.Hebrew Then 'Never used in practice
                        intSpring = cHebrew.BeforeOnAfter(.intDayOfWeek, .intDate, .intWeekNo, .intMonthNo, dblFix, blnValid)
                        If Not blnValid Then Exit Function
                    Else
                        Err.Raise(vbObjectError + 1, "cLocalTime.GetTZ", "invalid clock-change date-type." & .enmDateType)
                    End If
                End With
                With udtLocation.udtTZ(intI).udtDSTEnd
                    If .enmDateType = Main.DateType.gregorian Then
                        intFall = cGregorian.BeforeOnAfter(.intDayOfWeek, .intDate, .intWeekNo, .intMonthNo, dblFix, blnValid, udtLocation.udtTZ(intI).blnAvoidFestivals)
                        If Not blnValid Then Exit Function
                    ElseIf .enmDateType = Main.DateType.Hebrew Then 'Used in Israel in the fall of 2003-2010
                        intFall = cHebrew.BeforeOnAfter(.intDayOfWeek, .intDate, .intWeekNo, .intMonthNo, dblFix, blnValid)
                        If Not blnValid Then Exit Function
                    Else
                        Err.Raise(vbObjectError + 1, "cLocalTime.GetTZ", "invalid clock-change date-type.")
                    End If
                End With
                If intSpring = 0 Then 'On the day when changing from winter to summer time
                    GetTZ = cDMS.FromDMS(udtTime.udtDms, blnValid)
                    If Not blnValid Then Exit Function
                    If GetTZ <= udtLocation.udtTZ(intI).udtDSTStart.intHour Then
                        GetTZ = udtLocation.udtTZ(intI).dblTZ 'Before the change
                    ElseIf GetTZ >= udtLocation.udtTZ(intI).udtDSTStart.intHour + 1 Then
                        GetTZ = udtLocation.udtTZ(intI).dblTZ + 1 'After the change
                    Else 'Within the hour skipped - set according to the manual parmeters and flag as within the hour skipped
                        If udtLocation.blnDST Then
                            GetTZ = udtLocation.udtTZ(intI).dblTZ + 1
                        Else
                            GetTZ = udtLocation.udtTZ(intI).dblTZ
                        End If
                        blnChangeOverJump = True 'Within the hour skipped on changing time
                    End If
                ElseIf intFall = 0 Then 'On the day when changing from summer to winter time
                    GetTZ = cDMS.FromDMS(udtTime.udtDms, blnValid)
                    If Not blnValid Then Exit Function
                    If GetTZ >= udtLocation.udtTZ(intI).udtDSTEnd.intHour Then
                        GetTZ = udtLocation.udtTZ(intI).dblTZ 'Before the change
                    ElseIf GetTZ <= udtLocation.udtTZ(intI).udtDSTEnd.intHour - 1 Then
                        GetTZ = udtLocation.udtTZ(intI).dblTZ + 1 'After the change
                    Else 'Within the hour that occurs twice
                        If blnSecondTime Then 'Use the parameter provided to choose wether it is the first or second occurence
                            GetTZ = udtLocation.udtTZ(intI).dblTZ
                        Else
                            GetTZ = udtLocation.udtTZ(intI).dblTZ + 1
                        End If
                    End If
                ElseIf udtLocation.udtTZ(intI).udtDSTStart.intMonthNo < udtLocation.udtTZ(intI).udtDSTEnd.intMonthNo Then  'Northern hemisphere
                    If intSpring = -1 Or intFall = 1 Then 'Winter
                        GetTZ = udtLocation.udtTZ(intI).dblTZ
                    Else 'Summer
                        GetTZ = udtLocation.udtTZ(intI).dblTZ + 1
                    End If
                Else 'Southern hemisphere
                    If intFall = -1 Or intSpring = 1 Then 'Summer
                        GetTZ = udtLocation.udtTZ(intI).dblTZ + 1
                    Else 'Winter
                        GetTZ = udtLocation.udtTZ(intI).dblTZ
                    End If
                End If
            End If
        End If
        If GetTZ < -14 Or GetTZ > 15 Then
            Err.Raise(vbObjectError + 1, "cLocalTime.GetTZ", "Magnitude of time zone too large: " & GetTZ)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Function FromLocal(ByVal udtTime As Mytime, ByRef blnValid As Boolean, ByVal enmType As Main.DateType, ByRef blnChangeOverJump As Boolean) As Double
        Dim cDMS As New clsDMS
        Dim cSun As New clsSun
        Dim dblTZ As Double 'Time zone in hours
        Dim cHebrew As New clsHebrew
        'Convert to fraction of day since last UTC midnight

        On Error GoTo EH
        blnValid = True
        'Determine time zone
        dblTZ = GetTZ(dblDate, blnValid, udtTime.blnSecondTime, blnChangeOverJump)
        If Not blnValid Then Exit Function
        'Convert time
        FromLocal = (frmTime.FromAMPM(udtTime, blnValid) - dblTZ) / 24
        If Not blnValid Then Exit Function
        If enmType = Main.DateType.Hebrew Or enmType = Main.DateType.Islamic Then
            Call cHebrew.AdjustTime(frmDate.ToLngDate(dblDate, blnValid, False), FromLocal, blnValid, udtTime.blnSecondTime)
            'May be secondtime in the Hebrew calendar if the sunset is later than the prevoius day
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub ToLocal(ByVal dblFix As Double, ByRef udtTimeIn As Mytime, ByRef blnValid As Boolean, ByRef blnChangeOverTime As Boolean)
        Dim dblTZ As Double 'Time zone in hours
        Dim cDMS As New clsDMS
        'Convert to local time

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cLocal.ToLocal", "Magnitude of dblDate too large: " & dblFix)
        End If
        udtTimeIn.enmTimeType = Main.TimeType.localt 'Set the time type to local
        dblTZ = GetTZ(dblFix, blnValid, udtTimeIn.blnSecondTime, blnChangeOverTime) 'Find the time zone
        If Not blnValid Then Exit Sub
        Call cDMS.ToDMS(dblMod((dblFix - Int(dblFix)) * 24 + dblTZ, 24), udtTimeIn.udtDms) 'Convert time and then to base 60
        If udtTimeIn.udtDms.intDegreesHours = 24 And udtTimeIn.udtDms.intMinutes = 0 And udtTimeIn.udtDms.dblSeconds = 0 Then 'allow 24:00:00 due to rounding
            udtTimeIn.udtDms.intDegreesHours = 0
        End If
        If udtTimeIn.udtDms.intDegreesHours > 23 Then
            Err.Raise(vbObjectError + 1, "cLocal.ToLocal", "Invalid udtTimeIn.udtDms.intDegreesHours: " & udtTimeIn.udtDms.intDegreesHours)
        End If
        Call frmTime.ToAMPM(udtTimeIn, blnValid)
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Sub StartEndOfDay(ByVal dblFix As Double, ByRef enmDateType As Main.DateType, ByRef dblStart As Double, ByRef dblEnd As Double, ByRef intDayShift As Integer, ByRef blnValid As Boolean, ByRef blnChangeOverJump As Boolean)
        'Get the start and end of the day according to Local time zone at time dblFix
        Dim cHebrew As New clsHebrew
        Dim dblTZ As Double 'Time zone in hours

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cLocalTime.StartEndOfDay", "Magnitude of dblDate too large: " & dblFix)
        End If
        intDayShift = 0
        If enmDateType < Main.DateType.Hebrew Then
            dblTZ = GetTZ(dblFix, blnValid, False, blnChangeOverJump) 'Time zone at dblfix
            dblStart = Int(dblFix + dblTZ / 24) - dblTZ / 24
            dblEnd = Int(dblFix + dblTZ / 24 + 1) - dblTZ / 24
            intDayShift = 0
        Else 'Defined by sunset in the Hebrew and Islamic calendars
            Call cHebrew.StartEndDay(dblFix, dblStart, dblEnd, blnValid, intDayShift)
        End If
        If (dblEnd < dblStart Or dblEnd > dblStart + 2.002) Then
            Err.Raise(vbObjectError + 1, "cLocalTime.StartEndOfDay", "Difference between dblEnd and dblStart out of range: " & dblEnd & " - " & dblStart & " = " & dblEnd - dblStart)
        End If
        If (dblStart > dblFix Or dblStart < dblFix - 1.05) Then
            Err.Raise(vbObjectError + 1, "cLocalTime.StartEndOfDay", "Difference between dblFix and dblStart out of range: " & dblFix & " - " & dblStart & " = " & dblFix - dblStart)
        End If
        If (dblEnd < dblFix Or dblEnd > dblFix + 1.05) Then
            Err.Raise(vbObjectError + 1, "cLocalTime.StartEndOfDay", "Difference between dblEnd and dblFix out of range: " & dblEnd & " - " & dblFix & " = " & dblEnd - dblFix)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function ZeroTime(ByVal dblFix As Double, ByRef blnValid As Boolean) As Double
        'Time when the local time is 0:00
        Dim dblTZ As Double 'Time zone in hours

        dblTZ = GetTZ(dblFix, blnValid, False, False)
        If Not blnValid Then Exit Function
        ZeroTime = -dblTZ / 24
    End Function
End Class