Option Strict Off
Option Explicit On
Friend Class clsMeanSolar
    '(C)Roy E. Hoffman 2000-2011
    'See Main.bas for terms and conditions of use
    'Routines related to mean solar time
	
    Friend Function FromMeanSolar(ByVal udtTime As Mytime, ByRef blnValid As Boolean, ByVal enmType As Main.DateType) As Double
        'Convert from mean solar time, result in days
        Dim cDMS As New clsDMS
        Dim cSun As New clsSun
        Dim cHebrew As New clsHebrew

        On Error GoTo EH
        blnValid = True
        'Direct conversion
        FromMeanSolar = (frmTime.FromAMPM(udtTime, blnValid) - udtLocation.dblLongitude / 15) / 24
        If Not blnValid Then Exit Function
        If udtTime.blnSecondTime Then FromMeanSolar = FromMeanSolar + 1
        'Adjust day if necessary for Hebrew/Islamic day
        If enmType = Main.DateType.Hebrew Or enmType = Main.DateType.Islamic Then
            Call cHebrew.AdjustTime(Int(dblDate), FromMeanSolar, blnValid, udtTime.blnSecondTime)
        ElseIf frmTime.opt2.Checked Then
            blnValid = False
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub ToMeanSolar(ByVal dblFix As Double, ByRef udtTimeIn As Mytime, ByRef blnValid As Boolean)
        'Convert time in days UTC to mean solar time
        Dim cDMS As New clsDMS

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cMeanSolar.ToMeanSolar", "Magnitude of dblDate too large: " & dblFix)
        End If
        udtTimeIn.enmTimeType = Main.TimeType.meansolar 'Set the time type to mean solar
        'Calculate the time
        Call cDMS.ToDMS(dblMod((dblFix - Int(dblFix)) * 24 + udtLocation.dblLongitude / 15, 24), udtTimeIn.udtDms)
        If udtTimeIn.udtDms.intDegreesHours = 24 And udtTimeIn.udtDms.intMinutes = 0 And udtTimeIn.udtDms.dblSeconds = 0 Then 'allow 24:00:00 due to rounding
            udtTimeIn.udtDms.intDegreesHours = 0
        End If
        If udtTimeIn.udtDms.intDegreesHours > 23 Then
            Err.Raise(vbObjectError + 1, "cMeanSolar.ToMeanSolar", "Invalid udtTimeIn.udtDms.intDegreesHours: " & udtTimeIn.udtDms.intDegreesHours)
        End If
        Call frmTime.ToAMPM(udtTimeIn, blnValid)
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Sub StartEndOfDay(ByVal dblFix As Double, ByVal enmDateType As Main.DateType, ByRef dblStart As Double, ByRef dblEnd As Double, ByRef intDayShift As Integer, ByRef blnValid As Boolean)
        'Get start and end of mean solar day
        Dim cHebrew As New clsHebrew

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cMeanSolar.StartEndOfDay", "Magnitude of dblDate too large: " & dblFix)
        End If
        intDayShift = 0
        If enmDateType < Main.DateType.Hebrew Then
            If udtLocation.dblLongitude > 180 Or udtLocation.dblLongitude <= -180 Then
                Err.Raise(vbObjectError + 1, "cMeanSolar.StartEndOfDay", "udtLocation.dblLongitude out of range: " & udtLocation.dblLongitude)
            End If
            dblStart = Int(dblFix + udtLocation.dblLongitude / 360) - udtLocation.dblLongitude / 360
            dblEnd = dblStart + 1
            intDayShift = 0 'Int(dblFix + udtLocation.dblLongitude / 360) - Int(dblFix)
        Else
            Call cHebrew.StartEndDay(dblFix, dblStart, dblEnd, blnValid, intDayShift)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function ZeroTime(ByVal dblFix As Double) As Double
        'Set the time of 0:00
        ZeroTime = -udtLocation.dblLongitude / 360
    End Function
End Class