Option Strict Off
Option Explicit On
Friend Class clsPlanet
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Deals with functions pertaining to planets
	'Some of the algorithms from
	'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 87
	
	Const AU As Integer = 149597870 'Astronomical unit in km
	Const TwoPi As Double = 6.28318530717959
	
    Friend Sub HelioPlanet(ByVal strPlanet As String, ByVal dblFix As Double, ByRef dblL As Double, ByRef dblB As Double, ByRef dblR As Double, ByRef blnValid As Boolean)
        'Calculate heliocentric coordinates of a planet
        'Added save routine to save repeat calculations
        'Checks if the parameters have been loaded and the value previously calculated
        'Loads the parameters and calls the calculation routine (helioplanet2) as required
        Dim cHBody As New clsHBody

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax + 2 Then 'allow leaway for calls near the limit
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet", "Magnitude of dblFix too large: " & dblFix)
        End If
        Select Case strPlanet
            Case "Mercury"
                If blnMercury And Not blnChangedInput And dblFix = udtMercurySave.Fix_Renamed And dblFix <> 0 Then
                    With udtMercurySave
                        dblL = .L
                        dblB = .B
                        dblR = .R
                    End With
                Else
                    If Not blnMercury Then 'Read the orbital parameters the first time
                        Call cHBody.ReadHBody(strPlanet, udtMercury, lngMoonLR, lngMoonB, udtStar)
                        blnMercury = True
                    End If
                    Call helioplanet2(udtMercury, dblFix, dblL, dblB, dblR, blnValid)
                    If Not blnValid Then Exit Sub
                    With udtMercurySave
                        .Fix_Renamed = dblFix
                        .L = dblL
                        .B = dblB
                        .R = dblR
                    End With
                End If
            Case "Venus"
                If blnVenus And Not blnChangedInput And dblFix = udtVenusSave.Fix_Renamed And dblFix <> 0 Then
                    With udtVenusSave
                        dblL = .L
                        dblB = .B
                        dblR = .R
                    End With
                Else
                    If Not blnVenus Then 'Read the orbital parameters the first time
                        Call cHBody.ReadHBody(strPlanet, udtVenus, lngMoonLR, lngMoonB, udtStar)
                        blnVenus = True
                    End If
                    Call helioplanet2(udtVenus, dblFix, dblL, dblB, dblR, blnValid)
                    If Not blnValid Then Exit Sub
                    With udtVenusSave
                        .Fix_Renamed = dblFix
                        .L = dblL
                        .B = dblB
                        .R = dblR
                    End With
                End If
            Case "Earth"
                If blnEarth And Not blnChangedInput And dblFix = udtEarthSave.Fix_Renamed And dblFix <> 0 Then
                    With udtEarthSave
                        dblL = .L
                        dblB = .B
                        dblR = .R
                    End With
                Else
                    If Not blnEarth Then 'Read the orbital parameters the first time
                        Call cHBody.ReadHBody(strPlanet, udtEarth, lngMoonLR, lngMoonB, udtStar)
                        blnEarth = True
                    End If
                    Call helioplanet2(udtEarth, dblFix, dblL, dblB, dblR, blnValid)
                    If Not blnValid Then Exit Sub
                    With udtEarthSave
                        .Fix_Renamed = dblFix
                        .L = dblL
                        .B = dblB
                        .R = dblR
                    End With
                End If
            Case "Mars"
                If blnMars And Not blnChangedInput And dblFix = udtMarsSave.Fix_Renamed And dblFix <> 0 Then
                    With udtMarsSave
                        dblL = .L
                        dblB = .B
                        dblR = .R
                    End With
                Else
                    If Not blnMars Then 'Read the orbital parameters the first time
                        Call cHBody.ReadHBody(strPlanet, udtMars, lngMoonLR, lngMoonB, udtStar)
                        blnMars = True
                    End If
                    Call helioplanet2(udtMars, dblFix, dblL, dblB, dblR, blnValid)
                    If Not blnValid Then Exit Sub
                    With udtMarsSave
                        .Fix_Renamed = dblFix
                        .L = dblL
                        .B = dblB
                        .R = dblR
                    End With
                End If
            Case "Jupiter"
                If blnJupiter And Not blnChangedInput And dblFix = udtJupiterSave.Fix_Renamed And dblFix <> 0 Then
                    With udtJupiterSave
                        dblL = .L
                        dblB = .B
                        dblR = .R
                    End With
                Else
                    If Not blnJupiter Then 'Read the orbital parameters the first time
                        Call cHBody.ReadHBody(strPlanet, udtJupiter, lngMoonLR, lngMoonB, udtStar)
                        blnJupiter = True
                    End If
                    Call helioplanet2(udtJupiter, dblFix, dblL, dblB, dblR, blnValid)
                    If Not blnValid Then Exit Sub
                    With udtJupiterSave
                        .Fix_Renamed = dblFix
                        .L = dblL
                        .B = dblB
                        .R = dblR
                    End With
                End If
            Case "Saturn"
                If blnSaturn And Not blnChangedInput And dblFix = udtSaturnSave.Fix_Renamed And dblFix <> 0 Then
                    With udtSaturnSave
                        dblL = .L
                        dblB = .B
                        dblR = .R
                    End With
                Else
                    If Not blnSaturn Then 'Read the orbital parameters the first time
                        Call cHBody.ReadHBody(strPlanet, udtSaturn, lngMoonLR, lngMoonB, udtStar)
                        blnSaturn = True
                    End If
                    Call helioplanet2(udtSaturn, dblFix, dblL, dblB, dblR, blnValid)
                    If Not blnValid Then Exit Sub
                    With udtSaturnSave
                        .Fix_Renamed = dblFix
                        .L = dblL
                        .B = dblB
                        .R = dblR
                    End With
                End If
            Case Else
                Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet", "Invalid planet name: " & strPlanet)
        End Select
        If dblL < 0 Or dblL >= 360 Then
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet", "Invalid dblL: " & dblL)
        End If
        If dblB < -90 Or dblB > 90 Then
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet", "Invalid dblB: " & dblB)
        End If
        If dblR < 0 Then
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet", "Invalid dblR: " & dblR)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Sub helioplanet2(ByVal udtPars As VSOP, ByVal dblFix As Double, ByRef dblL As Double, ByRef dblB As Double, ByRef dblR As Double, ByRef blnValid As Boolean)
        'Calculate the heliocentric position of a planet
        Dim dblTau As Double 'Date in Julian ephermerides millenia from J2000
        Dim intI As Integer 'Parameter counter
        Dim intJ As Integer 'Index counter
        Dim dblAngle As Double 'Resulting angle
        Dim cJDE As New clsJDE

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax + 2 Then 'allow leaway for calls near the limit
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet2", "Magnitude of dblFix too large: " & dblFix)
        End If
        dblTau = (cJDE.fixed_jde(dblFix, blnValid) - 2451545) / 365250
        dblL = 0
        For intI = udtPars.NL To 0 Step -1
            dblL = dblL * dblTau
            For intJ = 1 To udtPars.L(intI).N
                'Reduce the angle to one circle (0 to 2 pi)
                'Written inline to increase speed
                dblAngle = udtPars.L(intI).B(intJ) + dblTau * udtPars.L(intI).C(intJ)
                dblAngle = dblAngle - TwoPi * Int(dblAngle / TwoPi)
                dblL = dblL + udtPars.L(intI).A(intJ) * System.Math.Cos(dblAngle)
            Next
        Next
        dblL = FracCirc(dblL / 100000000.0# * RADCON)
        dblB = 0
        For intI = udtPars.NB To 0 Step -1
            dblB = dblB * dblTau
            For intJ = 1 To udtPars.B(intI).N
                dblAngle = udtPars.B(intI).B(intJ) + dblTau * udtPars.B(intI).C(intJ)
                dblAngle = dblAngle - TwoPi * Int(dblAngle / TwoPi)
                dblB = dblB + udtPars.B(intI).A(intJ) * System.Math.Cos(dblAngle)
            Next
        Next
        dblB = FracCirc(dblB / 100000000.0# * RADCON)
        If dblB >= 180 Then dblB = dblB - 360
        dblR = 0
        For intI = udtPars.NR To 0 Step -1
            dblR = dblR * dblTau
            For intJ = 1 To udtPars.R(intI).N
                dblAngle = udtPars.R(intI).B(intJ) + dblTau * udtPars.R(intI).C(intJ)
                dblAngle = dblAngle - TwoPi * Int(dblAngle / TwoPi)
                dblR = dblR + udtPars.R(intI).A(intJ) * System.Math.Cos(dblAngle)
            Next
        Next
        dblR = dblR / 100000000.0# * 149597870
        If dblL < 0 Or dblL >= 360 Then
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet2", "Invalid dblL: " & dblL)
        End If
        If dblB < -90 Or dblB > 90 Then
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet2", "Invalid dblB: " & dblB)
        End If
        If dblR < 0 Then
            Err.Raise(vbObjectError + 1, "cPlanet.HelioPlanet2", "Invalid dblR: " & dblR)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
End Class