﻿Public Class clsReadResource
    'Resource text file.
    'The file is stored as a text file in the projects resources
    'A ' defines a comment
    'and it and subsequent characters are ignored
    'A _ defines a continuation to the next line
    'A , separates entries if that option is selected
    'Repeating a special character overrides the character
    'For each language, labels are first
    'then messages, languages, standards, language ordering and standards ordering
    'The ordering is defined such that the number of the nth listed item refers
    'to the order in English
    Friend Sub ReadResource(ByVal strResource As String, ByRef strLines() As String, ByRef intLine As Integer, Optional ByVal chrNotControl As Char = "_"c, _
                            Optional ByVal chrComment As Char = "'"c, Optional ByVal blnDivideLines As Boolean = False, Optional ByVal chrDivider As Char = ","c)
        Dim strText As String 'The text of the resource
        Dim strEntries() As String 'The text split into entries/lines
        Dim intEntries As Integer 'Number of entries
        Dim strLines2() As String 'Entries stored if dividing lines
        Dim intLines2 As Integer 'Number of entries stored if dividing lines
        Dim intI As Integer 'Line counter
        Dim intJ As Integer 'Entry counter

        strText = strResource 'Load the resource file as a string
        'strText = My.Resources.txtLocalization 'Load the resource file as a string
        Call DivideFile(strText, strLines, intLine, chrNotControl, chrComment) 'Divide it up into lines after removing comments and concatinating
        strText = "" 'Clear the string to release memory
        If blnDivideLines Then
            ReDim strEntries(0)
            ReDim strLines2(0)
            intLines2 = 0
            For intI = 0 To intLine
                Call DivideLine(strLines(intI), strEntries, intEntries, chrDivider, chrNotControl)
                ReDim Preserve strLines2(intLines2 + intEntries)
                For intJ = 0 To intEntries - 1
                    strLines2(intLines2 + intJ) = strEntries(intJ)
                Next
                intLines2 = intLines2 + intJ
            Next
            ReDim strLines(intLines2)
            intLines2 = intLines2
            For intI = 0 To intLines2 - 1
                strLines(intI) = strLines2(intI)
            Next
            intLine = intLines2
            Erase strLines2
        End If
    End Sub
    Private Sub DivideFile(ByRef strText As String, ByRef strLines() As String, ByRef intLine As Integer, ByVal chrConcat As Char, ByVal chrComment As Char)
        'Divide a resource file into strings removing comments and concatinating where indicated
        Dim intChr As Integer 'Character counter
        Dim blnEndFile As Boolean 'Indicates the end of file

        'Initialize variables
        ReDim strLines(0)
        intLine = 0
        intChr = 0
        blnEndFile = False
        'Read whole (including concatinated) lines from the file
        While Not blnEndFile
            strLines(intLine) = "" 'Initialize the line
            While strLines(intLine) = "" And Not blnEndFile 'Continue until the line contains something or the end of the file
                strLines(intLine) = ReadWholeLine(strText, intLine, intChr, chrConcat, chrComment) 'Read a line including concatination
                blnEndFile = (intChr >= Len(strText) - 1) 'Check for end of file
            End While
            If Not blnEndFile Then 'Add another line if not at the end of the file
                intLine = intLine + 1
                ReDim Preserve strLines(intLine)
            End If
        End While
        If strLines(intLine) = "" And intLine > 0 Then 'Reduce the number of lines if possible and the last one is empty
            intLine = intLine - 1
            ReDim Preserve strLines(intLine)
        End If
    End Sub
    Private Function ReadWholeLine(ByRef strText As String, ByVal intLine As Integer, ByRef intChr As Integer, ByVal chrNotControl As Char, ByVal chrComment As Char) As String
        'Read a line including concatination
        Dim CurrentLine As String
        Dim blnConcatinate As Boolean
        Dim intConcat As Integer
        Dim intI As Integer

        blnConcatinate = True
        CurrentLine = ReadLine(strText, intLine, intChr, chrComment, chrNotControl) 'Read a line and remove the comment
        While blnConcatinate
            intConcat = 0 'Count the number of concatination characters at the end
            If Len(CurrentLine) - intConcat > 0 Then
                While CurrentLine(Len(CurrentLine) - intConcat - 1) = chrNotControl And Len(CurrentLine) - intConcat > 0
                    intConcat = intConcat + 1
                End While
            End If
            blnConcatinate = (intConcat Mod 2 = 1) 'Concatinate if it is an odd number
            For intI = 1 To intConcat \ 2 'Half the even number of concatination markers that are really part of the text
                CurrentLine = Left(CurrentLine, Len(CurrentLine) - 1)
            Next
            If blnConcatinate Then 'Add another line if concatinating
                CurrentLine = String.Concat(Left(CurrentLine, Len(CurrentLine) - 1), vbCrLf, ReadLine(strText, intLine, intChr, chrComment, chrNotControl))
            End If
        End While
        ReadWholeLine = CurrentLine 'Return the result
    End Function
    Private Function ReadLine(ByRef strText As String, ByVal intLine As Integer, ByRef intChr As Integer, ByVal chrComment As Char, ByRef chrNotControl As Char) As String
        'Read a line and remove the comment
        Dim intStartLine As Integer
        Dim CurrentLine As String
        Dim intI As Integer

        intStartLine = intChr
        If intChr >= Len(strText) - 1 Then 'Return if end of file
            ReadLine = ""
            Exit Function
        End If
        While strText(intChr) <> Chr(13) And strText(intChr) <> Chr(10) And intChr < Len(strText) - 1 'Find the end of the line
            intChr = intChr + 1
        End While
        If intChr = Len(strText) - 1 And strText(intChr) <> Chr(13) And strText(intChr) <> Chr(10) Then intChr = intChr + 1 'Read last character if the end of file is not a return
        CurrentLine = Mid(strText, intStartLine + 1, intChr - intStartLine) 'Extract the line
        'Remove comment
        intI = 0
        While intI < Len(CurrentLine) - 1 'Scan for comment markers
            If CurrentLine(intI) = chrComment Or CurrentLine(intI) = chrNotControl Then
                If Len(CurrentLine) > intI + 2 Then 'Skip double markers
                    If CurrentLine(intI) = chrNotControl And CurrentLine(intI + 1) = " " Then
                        intI = intI + 1 'leave the notcontrol in so that leading spaces can be protected later
                    ElseIf CurrentLine(intI + 1) = chrComment Then
                        intI = intI + 1
                        CurrentLine = Left(CurrentLine, intI - 1) & Right(CurrentLine, Len(CurrentLine) - intI)
                    End If
                    'Else
                    'If Len(CurrentLine) > intI + 1 Then
                    'intI = intI + 1 'leave noncontrol at the end of a line
                    'End If
                End If
            End If
            If CurrentLine(intI) = chrComment And Len(CurrentLine) = intI + 2 Then
                If CurrentLine(intI + 1) = chrComment Then
                    intI = intI + 1
                End If
            End If
            If CurrentLine(intI) = chrComment Then 'Remove the comment but leave noncontrol at the end of a line
                CurrentLine = Left(CurrentLine, intI)
            Else
                intI = intI + 1
            End If
        End While
        If Len(CurrentLine) > intI Then
            If CurrentLine(intI) = chrComment Then
                CurrentLine = Left(CurrentLine, intI)
            End If
        End If
        'Collapse double comment markers into signle ones
        While intI < Len(CurrentLine) - 1
            If CurrentLine(intI) = chrComment Then
                CurrentLine = Left(CurrentLine, intI) & Right(CurrentLine, Len(CurrentLine - 1 - intI))
            End If
        End While
        ReadLine = CurrentLine
        intChr = intChr + 2 'Move the marker to after the CRLF
    End Function
    Friend Function ExtractLine(ByVal strText As String, ByVal intLineNo As Integer) As String
        'Find line
        Dim intI As Integer
        Dim intChr As Integer

        intI = 0
        intChr = 0
        While intI < intLineNo
            While strText(intChr) = vbCrLf And intChr < Len(strText) - 1 'skip empty line
                intChr = intChr + 1
            End While
            If Len(strText) - 1 > intChr Then
                If strText(intChr) = "'" Then
                    If Len(strText) - 1 > intChr + 1 Then
                        If strText(intChr + 1) <> "'" Then
                            While strText(intChr) <> vbCrLf And intChr < Len(strText) - 1
                                intChr = intChr + 1
                            End While
                            If strText(intChr) = vbCrLf Then
                                intI = intI + 1
                                intChr = intChr + 1
                            End If
                        End If
                    End If
                End If
            End If
        End While
        ExtractLine = ""
    End Function
    Private Sub DivideLine(ByRef strLine As String, ByRef strEntries() As String, ByRef intEntries As Integer, ByVal chrDivider As Char, ByRef chrNonControl As Char)
        'Divied a whole line into entries after concatination and removal of the comment
        Dim intLineChr As Integer
        Dim intStartChr As Integer
        Dim intI As Integer

        intStartChr = 0
        intLineChr = 0
        intEntries = 0
        ReDim strEntries(0)
        'Remove comment
        intI = 0
        While intLineChr < Len(strLine) 'Scan for divider markers
            'intStartChr = intLineChr
            If strLine(intLineChr) = chrNonControl Then
                If Len(strLine) > intLineChr + 2 Then 'Skip check for divider
                    intLineChr = intLineChr + 2
                Else
                    intLineChr = Len(strLine) 'If skipping brings it to the end of the line then move to the end of the line
                End If
            Else
                intLineChr = intLineChr + 1
            End If
            If intLineChr < Len(strLine) Then
                If strLine(intLineChr) = chrDivider Then 'Split the line
                    intEntries = intEntries + 1
                    ReDim Preserve strEntries(intEntries)
                    strEntries(intEntries - 1) = Mid(strLine, intStartChr + 1, intLineChr - intStartChr)
                    'RemoveNonControls(strEntries(intEntries - 1), chrnoncontrol)
                    intStartChr = intLineChr + 1
                    intLineChr = intLineChr + 1
                End If
            Else
                intEntries = intEntries + 1
                ReDim Preserve strEntries(intEntries)
                strEntries(intEntries - 1) = Mid(strLine, intStartChr + 1, intLineChr - intStartChr)
                'RemoveNonControls(strEntries(intEntries - 1), chrNonControl)
                intStartChr = intLineChr
            End If
        End While
        'Remove leading spaces
        For intI = 0 To intEntries - 1
            If Len(strEntries(intI)) > 1 Then
                While (strEntries(intI)(0) = " " Or strEntries(intI)(0) = vbTab) And Len(strEntries(intI)) > 1
                    strEntries(intI) = Right(strEntries(intI), Len(strEntries(intI)) - 1)
                End While
            End If
            If strEntries(intI) = " " Or strEntries(intI) = vbTab Then strEntries(intI) = ""
            RemoveNonControls(strEntries(intI), chrNonControl) ' Remove non-control characters after stripping leading spaces
        Next
    End Sub
    Function NextCharacter(ByVal strLine As String, ByVal intChar As Integer) As Char
        'Return the next character without giving an error and the end of line/file
        If intChar = Len(strLine) - 1 Then
            NextCharacter = Chr(13)
        Else
            NextCharacter = strLine(intChar + 1)
        End If
    End Function
    Function PreviousCharacter(ByVal strLine As String, ByVal intChar As Integer) As Char
        'Return the previous character without giving an error at the beginning of the line/file
        If intChar = 0 Then
            PreviousCharacter = Chr(13)
        Else
            PreviousCharacter = strLine(intChar - 1)
        End If
    End Function
    Sub RemoveNonControls(ByRef strLine As String, ByVal chrNonControl As Char)
        'Remove markers indicating that the subsequent character is not a control character
        Dim intI As Integer

        intI = 0
        While intI < Len(strLine) - 1
            If strLine(intI) = chrNonControl Then
                If NextCharacter(strLine, intI) = chrNonControl Then
                    strLine = Left(strLine, intI) & Right(strLine, Len(strLine) - intI - 1)
                    intI = intI + 2
                Else
                    strLine = Left(strLine, intI) & Right(strLine, Len(strLine) - intI - 1)
                    intI = intI + 1
                End If
            Else
                intI = intI + 1
            End If
        End While
    End Sub
End Class
