Option Strict Off
Option Explicit On
Friend Class clsSeasonal
	'(C)Roy E. Hoffman 2000
	'See Main.bas for terms and conditions of use
	
	'Seasonal time
	'0-12: Apparent Sunset to Apparent Sunrise
	'12-24: Apparent Sunrise to Apparent Sunset
    Friend Function FixToSeasonal(ByVal dblFix As Double, ByRef blnValid As Boolean) As Double
        'Convert UTC time in days to seasonal time as a fraction of the day
        Dim cHBody As New clsHBody
        Dim dblRiseTime As Double 'Sunrise time
        Dim dblSetTime As Double 'Sunset time
        Dim intDayShift As Short 'Number of days to shift in order to match the date
        Dim blnNeverRises As Boolean
        Dim blnNeverSets As Boolean

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cSeasonal.FixToSeasonal", "Magnitude of dblDate too large: " & dblFix)
        End If
        intDayShift = 0
        'Determine the sunrise time
        Call cHBody.RiseSet("Sun", Int(dblFix), False, dblRiseTime, True, blnNeverRises, blnNeverSets, blnValid)
        If blnNeverRises Then 'Use a meridian if there is no sunrise/set
            Call cHBody.Meridian("Sun", Int(dblFix), 0, dblSetTime, blnValid)
            If Not blnValid Then Exit Function
            Call cHBody.Meridian("Sun", Int(dblFix + 1), 0, dblRiseTime, blnValid)
            If Not blnValid Then Exit Function
            While dblFix < dblRiseTime And dblFix < dblSetTime
                dblRiseTime = dblSetTime
                Call cHBody.Meridian("Sun", Int(dblFix) + intDayShift - 1, 0, dblSetTime, blnValid)
                If Not blnValid Then Exit Function
                intDayShift = intDayShift - 1
            End While
            While dblFix > dblRiseTime And dblFix > dblSetTime
                dblSetTime = dblRiseTime
                Call cHBody.Meridian("Sun", Int(dblFix) + intDayShift + 2, 0, dblRiseTime, blnValid)
                If Not blnValid Then Exit Function
                intDayShift = intDayShift + 1
            End While
        ElseIf blnNeverSets Then
            Call cHBody.Meridian("Sun", Int(dblFix), 180, dblRiseTime, blnValid)
            If Not blnValid Then Exit Function
            Call cHBody.Meridian("Sun", Int(dblFix + 1), 180, dblSetTime, blnValid)
            If Not blnValid Then Exit Function
            While dblFix < dblRiseTime And dblFix < dblSetTime
                dblSetTime = dblRiseTime
                Call cHBody.Meridian("Sun", Int(dblFix) + intDayShift - 1, 180, dblRiseTime, blnValid)
                If Not blnValid Then Exit Function
                intDayShift = intDayShift - 1
            End While
            While dblFix > dblRiseTime And dblFix > dblSetTime
                dblRiseTime = dblSetTime
                Call cHBody.Meridian("Sun", Int(dblFix) + intDayShift + 2, 180, dblSetTime, blnValid)
                If Not blnValid Then Exit Function
                intDayShift = intDayShift + 1
            End While
        Else
            If Not blnValid Then Exit Function
            'Find sunset
            Call cHBody.RiseSet("Sun", Int(dblFix), True, dblSetTime, True, blnNeverRises, blnNeverSets, blnValid)
            If Not blnValid Then Exit Function
            'Shift the day/night as necessary
            While dblFix < dblRiseTime And dblFix < dblSetTime
                Call cHBody.RiseSet("Sun", Int(dblFix) + intDayShift - 1, True, dblSetTime, True, blnNeverRises, blnNeverSets, blnValid)
                If Not blnValid Then Exit Function
                If dblFix < dblSetTime Then
                    intDayShift = intDayShift - 1
                    Call cHBody.RiseSet("Sun", Int(dblFix) + intDayShift, False, dblRiseTime, True, blnNeverRises, blnNeverSets, blnValid)
                    If Not blnValid Then Exit Function
                End If
            End While
            While dblFix > dblSetTime And dblFix > dblRiseTime
                Call cHBody.RiseSet("Sun", Int(dblFix) + intDayShift + 1, False, dblRiseTime, True, blnNeverRises, blnNeverSets, blnValid)
                If Not blnValid Then Exit Function
                If dblFix > dblRiseTime Then
                    intDayShift = intDayShift + 1
                    Call cHBody.RiseSet("Sun", Int(dblFix) + intDayShift, True, dblSetTime, True, blnNeverRises, blnNeverSets, blnValid)
                    If Not blnValid Then Exit Function
                End If
            End While
        End If
        'Convert to seasonal time
        If dblRiseTime > dblSetTime Then
            FixToSeasonal = (dblFix - dblSetTime) / (dblRiseTime - dblSetTime) / 2
        Else
            FixToSeasonal = (dblFix - dblRiseTime) / (dblSetTime - dblRiseTime) / 2 + 0.5
        End If
        If FixToSeasonal = 1 Then 'Deal with limiting case
            FixToSeasonal = 0
        End If
        If FixToSeasonal < 0 Or FixToSeasonal >= 1 Then
            Err.Raise(vbObjectError + 1, "cSeasonal.FixToSeasonal", "FixToSeasonal out of range: " & FixToSeasonal)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub ToSeasonal(ByVal dblFix As Double, ByRef udtTimeIn As Mytime, ByRef blnValid As Boolean)
        'Convert the fraction of a day to the time format
        Dim dblTimeIn As Double 'Time structure for the result
        Dim cDMS As New clsDMS

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cSeasonal.ToSeasonal", "Magnitude of dblDate too large: " & dblFix)
        End If
        dblTimeIn = FixToSeasonal(dblFix, blnValid) 'Obtain the time as a fraction of the day
        If Not blnValid Then Exit Sub
        udtTimeIn.bln24Hr = False 'Use night/day mode
        udtTimeIn.blnPM = dblTimeIn >= 0.5 'Determine if it isduring the day
        udtTimeIn.enmTimeType = Main.TimeType.seasonal
        Call cDMS.ToDMS(dblTimeIn * 24, udtTimeIn.udtDms) 'Convert to base 60
        Call frmTime.ToAMPM(udtTimeIn, blnValid)
        If udtTimeIn.udtDms.intDegreesHours > 12 Or udtTimeIn.udtDms.intDegreesHours = 12 And udtTimeIn.udtDms.intMinutes > 0 Then
            Err.Raise(vbObjectError + 1, "cSeasonal.ToSeasonal", "Invalid udtTimeIn.udtDms.intDegreesHours: " & udtTimeIn.udtDms.intDegreesHours)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function FromSeasonal(ByVal udtTime As Mytime, ByRef blnValid As Boolean) As Double
        'Convert seasonal time to UTC time in days
        Dim cHBody As New clsHBody
        Dim cDMS As New clsDMS
        Dim dblRiseTime As Double
        Dim dblSetTime As Double
        Dim blnNeverRises As Boolean
        Dim blnNeverSets As Boolean

        On Error GoTo EH
        blnValid = True
        If frmTime.FromAMPM(udtTime, blnValid) < 12 Then
            If udtTime.blnSecondTime Then
                Call cHBody.RiseSet("Sun", Int(dblDate), True, dblSetTime, True, blnNeverRises, blnNeverSets, blnValid)
                If Not blnValid Then Exit Function
                Call cHBody.RiseSet("Sun", Int(dblDate + 1), False, dblRiseTime, True, blnNeverRises, blnNeverSets, blnValid)
                If Not blnValid Then Exit Function
                FromSeasonal = frmTime.FromAMPM(udtTime, blnValid) / 12 * (dblRiseTime - dblSetTime) + dblSetTime
                If Not blnValid Then Exit Function
            Else
                Call cHBody.RiseSet("Sun", Int(dblDate - 1), True, dblSetTime, True, blnNeverRises, blnNeverSets, blnValid)
                If Not blnValid Then Exit Function
                Call cHBody.RiseSet("Sun", Int(dblDate), False, dblRiseTime, True, blnNeverRises, blnNeverSets, blnValid)
                If Not blnValid Then Exit Function
                FromSeasonal = frmTime.FromAMPM(udtTime, blnValid) / 12 * (dblRiseTime - dblSetTime) + dblSetTime
                If Not blnValid Then Exit Function
            End If
        Else
            Call cHBody.RiseSet("Sun", Int(dblDate), True, dblSetTime, True, blnNeverRises, blnNeverSets, blnValid)
            If Not blnValid Then Exit Function
            Call cHBody.RiseSet("Sun", Int(dblDate), False, dblRiseTime, True, blnNeverRises, blnNeverSets, blnValid)
            If Not blnValid Then Exit Function
            FromSeasonal = (frmTime.FromAMPM(udtTime, blnValid) - 12) / 12 * (dblSetTime - dblRiseTime) + dblRiseTime
            If Not blnValid Then Exit Function
        End If
        FromSeasonal = FromSeasonal - Int(dblDate)
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
End Class