Option Strict Off
Option Explicit On
Friend Class clsTrueSolar
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Deals with true solar time
	
    Friend Function FromTrueSolar(ByVal udtTime As Mytime, ByRef blnValid As Boolean, ByVal enmType As Main.DateType) As Double
        'Convert to UTC time in days from true solar time
        Dim cDMS As New clsDMS
        Dim udtSun As HBody 'Parameters for the Sun
        Dim cSun As New clsSun
        Dim udtTimeIn As Mytime 'Copy of time given
        Dim udtTimeOut As Mytime 'Result of iterated conversion
        Dim cMeanSolar As New clsMeanSolar
        Dim dblDiff As Double 'Change upon iteration
        Dim dblTime As Double 'Time in days
        Dim blnNeverRises As Boolean
        Dim blnNeverSets As Boolean
        Dim cHebrew As New clsHebrew
        Dim dblOldDiff As Double 'Previous change upon iteration

        On Error GoTo EH
        blnValid = True
        'Copy the time given before iteration
        udtTimeIn.enmTimeType = Main.TimeType.meansolar
        udtTimeIn.bln24Hr = udtTime.bln24Hr
        udtTimeIn.blnPM = udtTime.blnPM
        udtTimeIn.blnSecondTime = udtTime.blnSecondTime
        udtTimeIn.udtDms.blnSign = True
        udtTimeIn.udtDms.dblSeconds = udtTime.udtDms.dblSeconds
        udtTimeIn.udtDms.intMinutes = udtTime.udtDms.intMinutes
        udtTimeIn.udtDms.intDegreesHours = udtTime.udtDms.intDegreesHours
        udtTimeOut.enmTimeType = Main.TimeType.truesolar
        'Set differences for iteration
        dblOldDiff = 1
        dblDiff = 0
        While System.Math.Abs(dblDiff - dblOldDiff) > 1 / 72000 'While not accurate to 0.05 s
            'Mean solar time in days
            dblTime = Frac((frmTime.FromAMPM(udtTimeIn, blnValid) - udtLocation.dblLongitude / 15) / 24)
            If Not blnValid Then Exit Function
            If udtTimeIn.blnSecondTime Then dblTime = dblTime + 1
            'Convert to true solar
            Call ToTrueSolar(dblDate + dblTime, udtTimeOut, blnValid)
            If Not blnValid Then Exit Function
            dblOldDiff = dblDiff 'Store the previous difference
            'Calculate the new difference in hours
            dblDiff = frmTime.FromAMPM(udtTime, blnValid) - frmTime.FromAMPM(udtTimeOut, blnValid)
            If Not blnValid Then Exit Function
            'The difference should only be a few minutes so remove any near-full days
            If dblDiff >= 12 Then
                dblDiff = dblDiff - 24
            ElseIf dblDiff < -12 Then
                dblDiff = dblDiff + 24
            End If
            'Convert to time format
            Call cDMS.ToDMS(frmTime.FromAMPM(udtTimeIn, blnValid) + dblDiff, udtTimeOut.udtDms)
            If Not blnValid Then Exit Function
            udtTimeOut.blnSecondTime = udtTimeIn.blnSecondTime
            Call frmTime.ToAMPM(udtTimeOut, blnValid)
            If Not blnValid Then Exit Function
        End While
        'Calculate the true solar time
        FromTrueSolar = Frac((frmTime.FromAMPM(udtTimeOut, blnValid) - udtLocation.dblLongitude / 15) / 24)
        If Not blnValid Then Exit Function
        If enmType = Main.DateType.Hebrew Or enmType = Main.DateType.Islamic Then
            Call cHebrew.AdjustTime(Int(dblDate), FromTrueSolar, blnValid, udtTime.blnSecondTime)
        ElseIf frmTime.opt2.Checked Then
            blnValid = False
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub ToTrueSolar(ByVal dblFix As Double, ByRef udtTimeIn As Mytime, ByRef blnValid As Boolean)
        'Convert UTC time in days to true solar time
        Dim udtSun As HBody
        Dim cHBody As New clsHBody
        Dim cDMS As New clsDMS
        Dim dblTimeIn As Double

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cTrueSolar.ToTrueSolar", "Magnitude of dblDate too large: " & dblFix)
        End If
        udtTimeIn.enmTimeType = Main.TimeType.truesolar
        udtTimeIn.bln24Hr = udtTime.bln24Hr
        udtTimeIn.blnPM = udtTime.blnPM
        Call cHBody.TrueHBody("Sun", dblFix, udtSun, blnValid)
        If Not blnValid Then Exit Sub
        dblTimeIn = udtSun.HourAngTopo / 360 + 0.5
        If dblTimeIn >= 1 Then
            dblTimeIn = dblTimeIn - 1
        ElseIf dblTimeIn < 0 Then
            dblTimeIn = dblTimeIn + 1
        End If
        Call cDMS.ToDMS(dblTimeIn * 24, udtTimeIn.udtDms)
        If udtTimeIn.udtDms.intDegreesHours > 24 Or udtTimeIn.udtDms.intDegreesHours = 24 And udtTimeIn.udtDms.intMinutes > 0 Then
            Err.Raise(vbObjectError + 1, "cTrueSolar.ToTrueSolar", "Invalid udtTimeIn.udtDms.intDegreesHours: " & udtTimeIn.udtDms.intDegreesHours)
        End If
        Call frmTime.ToAMPM(udtTimeIn, blnValid)
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Sub StartEndOfDay(ByVal dblFix As Double, ByVal enmDateType As Main.DateType, ByRef dblStart As Double, ByRef dblEnd As Double, ByRef intDayShift As Integer, ByRef blnValid As Boolean)
        'Get start and end of true solar day (true midnight unless Hebrew or Islamic calendar)
        'The calling routine must shift the day using intDayShift if necessary
        Dim cHebrew As New clsHebrew
        Dim cHBody As New clsHBody

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cTrueSolar.StartEndOfDay", "Magnitude of dblDate too large: " & dblFix)
        End If
        intDayShift = 0
        If enmDateType < Main.DateType.Hebrew Then
            Call cHBody.Meridian("Sun", Int(dblFix), -180, dblStart, blnValid, -180)
            Call cHBody.Meridian("Sun", Int(dblFix), 180, dblEnd, blnValid, 180)
            If dblStart < Int(dblFix) - 0.5 - udtLocation.dblLongitude / 360 Or dblStart > Int(dblFix) + 0.5 - udtLocation.dblLongitude / 360 Then
                Err.Raise(vbObjectError + 1, "cTrueSolar.StartEndOfDay", "DblStart out of range: " & dblStart & " where dblFix = " & dblFix)
            End If
            If dblEnd < Int(dblFix) + 0.5 - udtLocation.dblLongitude / 360 Or dblStart > Int(dblFix) + 1.5 - udtLocation.dblLongitude / 360 Then
                Err.Raise(vbObjectError + 1, "cTrueSolar.StartEndOfDay", "DblStart out of range: " & dblStart & " where dblFix = " & dblFix)
            End If
        Else
            Call cHebrew.StartEndDay(dblFix, dblStart, dblEnd, blnValid, intDayShift)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function ZeroTime(ByVal dblFix As Double, ByRef blnValid As Boolean) As Double
        'Time of 0:00 - not the start of the day for Hebrew and Islamic calendars
        Dim cHBody As New clsHBody
        Dim intDayShift As Short
        Dim dblEnd As Double

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cTrueSolar.ZeroTime", "Magnitude of dblDate too large: " & dblFix)
        End If
        If udtLocation.dblLongitude > 180 Or udtLocation.dblLongitude <= -180 Then
            Err.Raise(vbObjectError + 1, "cTrueSolar.ZeroTime", "udtLocation.dblLongitude out of range: " & udtLocation.dblLongitude)
        End If
        intDayShift = 0
        ZeroTime = 0
        Call cHBody.Meridian("Sun", Int(dblFix), -180, ZeroTime, blnValid, -180)
        ZeroTime = ZeroTime - Int(dblFix + udtLocation.dblLongitude / 360)
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
End Class