Option Strict Off
Option Explicit On
Friend Class clsUTC
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Deals with UTC -- uniersal coordinated time
    Friend Function ZeroTime(ByVal dblFix As Double) As Double
        'Calculate the time of 0:00 - for UTC this is 0
        ZeroTime = 0
    End Function
    Friend Function FromUTC(ByVal udtTime As Mytime, ByRef blnValid As Boolean, ByVal enmType As Main.DateType, Optional ByVal blnAllow2ndTime As Boolean = False) As Double
        Dim cDMS As New clsDMS
        Dim cSun As New clsSun
        Dim dblSet As Double 'Sunset time
        Dim blnNeverRises As Boolean
        Dim blnNeverSets As Boolean
        Dim intDayShift As Short
        Dim cHebrew As New clsHebrew

        'Convert UTC to fraction of day
        On Error GoTo EH
        blnValid = True
        FromUTC = frmTime.FromAMPM(udtTime, blnValid) / 24
        If Not blnValid Then Exit Function
        If udtTime.blnSecondTime Then FromUTC = FromUTC + 1
        If enmType = Main.DateType.Hebrew Or enmType = Main.DateType.Islamic Then
            Call cHebrew.AdjustTime(Int(dblDate), FromUTC, blnValid, udtTime.blnSecondTime)
        ElseIf udtTime.blnSecondTime And Not blnAllow2ndTime Then
            Err.Raise(vbObjectError + 1, "cUTC.FromUTC", "blnSecondTime cannot occur with udtDateType: " & udtDate.enmDateType)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub ToUTC(ByVal dblFix As Double, ByRef udtTimeIn As Mytime, ByRef blnValid As Boolean)
        Dim cDMS As New clsDMS
        'Convert fixed radix to UTC

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cUTC.ToUTC", "Magnitude of dblDate too large: " & dblFix)
        End If
        udtTimeIn.enmTimeType = Main.TimeType.utc
        Call cDMS.ToDMS((dblFix - Int(dblFix)) * 24, udtTimeIn.udtDms)
        udtTimeIn.blnSecondTime = False
        If udtTimeIn.udtDms.intDegreesHours > 23 Then
            Err.Raise(vbObjectError + 1, "cUTC.ToUTC", "Invalid udtTimeIn.udtDms.intDegreesHours: " & udtTimeIn.udtDms.intDegreesHours)
        End If
        Call frmTime.ToAMPM(udtTimeIn, blnValid)
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Sub StartEndOfDay(ByVal dblFix As Double, ByVal enmDateType As Main.DateType, ByRef dblStart As Double, ByRef dblEnd As Double, ByRef blnValid As Boolean, ByRef intDayShift As Integer)
        Dim cHebrew As New clsHebrew

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cUTC.StartEndOfDay", "Magnitude of dblDate too large: " & dblFix)
        End If
        intDayShift = 0
        If enmDateType < Main.DateType.Hebrew Then
            dblStart = Int(dblFix)
            dblEnd = dblStart + 1
        Else
            Call cHebrew.StartEndDay(dblFix, dblStart, dblEnd, blnValid, intDayShift)
            If Not blnValid Then Exit Sub
        End If
        If (dblEnd < dblStart Or dblEnd > dblStart + 2.002) Then
            Err.Raise(vbObjectError + 1, "cUTC.StartEndOfDay", "Difference between dblEnd and dblStart out of range: " & dblEnd & " - " & dblStart & " = " & dblEnd - dblStart)
        End If
        If (dblStart > dblFix Or dblStart < dblFix - 1.3) Then
            Err.Raise(vbObjectError + 1, "cUTC.StartEndOfDay", "Difference between dblFix and dblStart out of range: " & dblFix & " - " & dblStart & " = " & dblFix - dblStart)
        End If
        If (dblEnd < dblFix Or dblEnd > dblFix + 1.3) Then
            Err.Raise(vbObjectError + 1, "cUTC.StartEndOfDay", "Difference between dblEnd and dblFix out of range: " & dblEnd & " - " & dblFix & " = " & dblEnd - dblFix)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
End Class