Option Strict Off
Option Explicit On
Friend Class clsSun
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Deals with Sun specific proceedures
	'Some of the algorithms from
	'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 87
	
	Const TPI As Double = 6.28318530717959
	Const PI As Double = 3.14159265358979
	Const RADCON As Double = 0.01745329252
	Const AU As Integer = 149597870 'Astronomical unit in km
    Friend Function MeanSun(ByVal dblFix As Double, ByRef blnValid As Boolean) As Double 'Longitude in degrees
        'Calculate the mean longitude of the Sun
        Dim cHBody As New clsHBody
        Dim dblT As Double

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax + 2 Then 'allow leaway for calls near the limit
            Err.Raise(vbObjectError + 1, "cHBody.RiseSet", "Magnitude of dblFix too large: " & dblFix)
        End If
        dblT = cHBody.ToJ2000(dblFix, blnValid)
        MeanSun = FracCirc(280.46646 + dblT * (36000.76983 + 0.0003032 * dblT))
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Function NextEqSol(ByVal dblFix As Double) As String
        'Find next vernal equinox
        Dim dblEqSol As Double
        Dim cHBody As New clsHBody
        Dim udtSun As HBody
        Dim dblDecl As Double
        Dim blnValid As Boolean

        dblEqSol = (360 - MeanSun(dblFix, blnValid)) * 365.242 / 360 + dblFix
        Call cHBody.TrueHBody("Sun", dblEqSol, udtSun, blnValid)
        dblDecl = udtSun.DeclGeo
        While System.Math.Abs(dblDecl) > 0.00001
            dblEqSol = dblEqSol - dblDecl / 0.398 * 365.242 / 360
            Call cHBody.TrueHBody("Sun", dblEqSol, udtSun, blnValid)
            dblDecl = udtSun.DeclGeo
        End While
        NextEqSol = strTexts(intListclsSunBase + intLanguage) & strTime(dblEqSol, CStr(0))
        'NextEqSol = "Next vernal equinox " & strTime(dblEqSol, CStr(0))
    End Function
End Class