Option Strict Off
Option Explicit On
Friend Class frmCriterion
    'Choose the criterion for lunar visiblity
    'In the non-release version the criterion can be tested while in this window
    Inherits System.Windows.Forms.Form
    Dim blnCancelled As Boolean
    Dim blnIsInitializing As Boolean
    Friend Sub SetDefaults() 'Of lunar visibility criterion
        udtCriterion.intType = 2 'DALT-width
        udtCriterion.intDarkness = 2 'DALT
        udtCriterion.intMeasTo = 0 'Center
        udtCriterion.intBrightness = 3 'Crescent width
        udtCriterion.intMeasFrom = 3 'Apparent
        udtCriterion.dblWhen = 0.6 'Best time 0.6 of lag after sunset
        udtCriterion.intWhen = 2 'Fraction of lag
    End Sub
    Function Evaluate(ByRef strError As String) As Boolean
        'Check that the parameters are valid

        Evaluate = False
        strError = ""
        If cboType.SelectedIndex = 0 Then 'If user defined, check parameters
            If (cboMoonBright.SelectedIndex < 3 Or cboMoonBright.SelectedIndex > 5) And cboFrom.SelectedIndex > 1 Then 'incompatibility
                Evaluate = True
                strError = strTexts(intfrmCriterionBase + 8 * intNLanguages + intLanguage)
            End If
            If Not IsNumeric(txtWhen.Text) Then
                If Not Evaluate Then
                    strError = strError & vbCrLf
                    Evaluate = True
                End If
                strError = strError & strTexts(intfrmCriterionBase + 9 * intNLanguages + intLanguage)
            Else
                Select Case cboWhen.SelectedIndex
                    Case 0 'Sun  below horizon
                        If CDbl(txtWhen.Text) < 0 Or CDbl(txtWhen.Text) > 10 Then
                            If Not Evaluate Then
                                strError = strError & vbCrLf
                                Evaluate = True
                            End If
                            strError = strError & strTexts(intfrmCriterionBase + 10 * intNLanguages + intLanguage)
                        End If
                    Case 1 'Mins after/before sunset/rise
                        If CDbl(txtWhen.Text) < 0 Or CDbl(txtWhen.Text) > 60 Then
                            If Not Evaluate Then
                                strError = strError & vbCrLf
                                Evaluate = True
                            End If
                            strError = strError & strTexts(intfrmCriterionBase + 11 * intNLanguages + intLanguage)
                        End If
                    Case 2 'Fraction of lag
                        If CDbl(txtWhen.Text) < 0 Or CDbl(txtWhen.Text) > 1 Then
                            If Not Evaluate Then
                                strError = strError & vbCrLf
                                Evaluate = True
                            End If
                            strError = strError & strTexts(intfrmCriterionBase + 12 * intNLanguages + intLanguage)
                        End If
                End Select
            End If
        End If
        If Not Evaluate Then
            udtCriterion.intType = cboType.SelectedIndex
            udtCriterion.intDarkness = cboDarkness.SelectedIndex
            udtCriterion.intMeasTo = cboMoonPart.SelectedIndex
            udtCriterion.intBrightness = cboMoonBright.SelectedIndex
            udtCriterion.intMeasFrom = cboFrom.SelectedIndex
            If IsNumeric(txtWhen.Text) Then udtCriterion.dblWhen = CDbl(txtWhen.Text)
            udtCriterion.intWhen = cboWhen.SelectedIndex
        End If
    End Function

    Private Sub cboDarkness_SelectedIndexChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cboDarkness.SelectedIndexChanged
        If Not blnIsInitializing Then
            cboMoonPart.Enabled = (cboType.SelectedIndex = 0 And cboDarkness.SelectedIndex = 2)
        End If
    End Sub

    Private Sub cboType_SelectedIndexChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cboType.SelectedIndexChanged
        If Not blnIsInitializing Then
            cboDarkness.Enabled = (cboType.SelectedIndex = 0)
            cboMoonPart.Enabled = (cboType.SelectedIndex = 0 And cboDarkness.SelectedIndex = 2)
            cboMoonBright.Enabled = (cboType.SelectedIndex = 0)
            cboFrom.Enabled = (cboType.SelectedIndex = 0)
            txtWhen.Enabled = (cboType.SelectedIndex = 0)
            cboWhen.Enabled = (cboType.SelectedIndex = 0)
        End If
    End Sub

    Private Sub cmdCancel_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdCancel.Click
        'If cancelled and changes have been made then confirm the choice
        If cboType.SelectedIndex <> udtCriterion.intType Or _
            cboDarkness.SelectedIndex <> udtCriterion.intDarkness Or _
            cboMoonPart.SelectedIndex <> udtCriterion.intMeasTo Or _
            cboMoonBright.SelectedIndex <> udtCriterion.intBrightness Or _
            cboFrom.SelectedIndex <> udtCriterion.intMeasFrom Or _
            txtWhen.Text <> CStr(udtCriterion.dblWhen) Or _
            cboWhen.SelectedIndex <> udtCriterion.intWhen Then 'If there are already changes then confirm cancel
            strMsgBox = strTexts(intStringBase + 5 * intNLanguages + intLanguage)
            MyMsgBox.ShowDialog()
            If MyMsgBox.DialogResult = System.Windows.Forms.DialogResult.OK Then
                'Discard changes and close
                blnCancelled = True
                Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
                Me.Close()
            End If
        Else
            'There were no changes so close anyway
            blnCancelled = True
            Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
            Me.Close()
        End If
    End Sub

    Private Sub cmdOK_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdOK.Click
        'Check the result and save and close if OK
        Dim strError As String

        strError = ""
        If Not Evaluate(strError) Then
            'The result is valid so close
            blnCancelled = True
            Me.DialogResult = System.Windows.Forms.DialogResult.OK
            Me.Close()
        Else
            'If there is an error, report it
            MsgBox(strError)
        End If
    End Sub

    Private Sub cmdTest_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdTest.Click
        'Test the criterion against a databas of observations
        'Not inculded in the release version
        Dim strError As String
        Dim cMoon As New clsMoon

        strError = ""
        If Not Evaluate(strError) Then
            Call cMoon.AnalyzeCriterion()
        Else
            MsgBox(strError)
        End If
    End Sub

    Private Sub MyRefresh()
        'Refresh the window after an action
        blnIsInitializing = True
        cboType.SelectedIndex = udtCriterion.intType
        blnIsInitializing = False
        cboType.Text = VB6.GetItemString(cboType, udtCriterion.intType)
        blnIsInitializing = True
        cboDarkness.Text = VB6.GetItemString(cboDarkness, udtCriterion.intDarkness)
        blnIsInitializing = False
        cboDarkness.Enabled = (udtCriterion.intType = 0)
        cboMoonPart.SelectedIndex = udtCriterion.intMeasTo
        cboMoonPart.Text = VB6.GetItemString(cboMoonPart, udtCriterion.intMeasTo)
        cboMoonPart.Enabled = (udtCriterion.intType = 0 And udtCriterion.intDarkness = 2)
        cboMoonBright.SelectedIndex = udtCriterion.intBrightness
        cboMoonBright.Text = VB6.GetItemString(cboMoonBright, udtCriterion.intBrightness)
        cboMoonBright.Enabled = (udtCriterion.intType = 0)
        cboFrom.SelectedIndex = udtCriterion.intMeasFrom
        cboFrom.Text = VB6.GetItemString(cboFrom, udtCriterion.intMeasFrom)
        cboFrom.Enabled = (udtCriterion.intType = 0)
        txtWhen.Text = CStr(udtCriterion.dblWhen)
        txtWhen.Enabled = (udtCriterion.intType = 0)
        cboWhen.SelectedIndex = udtCriterion.intWhen
        cboWhen.Text = VB6.GetItemString(cboWhen, udtCriterion.intWhen)
    End Sub
    Private Sub frmCriterion_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        'Load the form and display all labels
        blnCancelled = False
        blnIsInitializing = True
        Me.Text = strTexts(intfrmCriterionBase + intLanguage)
        lblType.Text = strTexts(intfrmCriterionBase + intNLanguages + intLanguage)
        lblDark.Text = strTexts(intfrmCriterionBase + 2 * intNLanguages + intLanguage)
        lblMoonPart.Text = strTexts(intfrmCriterionBase + 3 * intNLanguages + intLanguage)
        lblMoonBright.Text = strTexts(intfrmCriterionBase + 4 * intNLanguages + intLanguage)
        lblFrom.Text = strTexts(intfrmCriterionBase + 5 * intNLanguages + intLanguage)
        lblWhen.Text = strTexts(intfrmCriterionBase + 6 * intNLanguages + intLanguage)
        cmdDefault.Text = strTexts(intStringBase + intLanguage)
        cmdOK.Text = strTexts(intStringBase + 2 * intNLanguages + intLanguage)
        cmdCancel.Text = strTexts(intStringBase + intNLanguages + intLanguage)
        cmdTest.Text = strTexts(intfrmCriterionBase + 7 * intNLanguages + intLanguage)
        cmdTest.Visible = Not ReleaseVersion
        cboType.Items.Clear()
        cboType.Items.Add(strTexts(intListfrmCriterionBase + intLanguage))
        cboType.Items.Add(strTexts(intListfrmCriterionBase + intNLanguages + intLanguage))
        cboType.Items.Add(strTexts(intListfrmCriterionBase + 2 * intNLanguages + intLanguage))
        cboType.Items.Add(strTexts(intListfrmCriterionBase + 3 * intNLanguages + intLanguage))
        cboType.Items.Add(strTexts(intListfrmCriterionBase + 4 * intNLanguages + intLanguage))
        cboType.Items.Add(strTexts(intListfrmCriterionBase + 5 * intNLanguages + intLanguage))
        cboType.Items.Add(strTexts(intListfrmCriterionBase + 6 * intNLanguages + intLanguage))
        blnIsInitializing = False 'Force an update
        cboType.Items.Add(strTexts(intListfrmCriterionBase + 7 * intNLanguages + intLanguage))
        blnIsInitializing = True
        cboDarkness.Items.Clear()
        cboDarkness.Items.Add(strTexts(intListfrmCriterionBase + 8 * intNLanguages + intLanguage))
        cboDarkness.Items.Add(strTexts(intListfrmCriterionBase + 9 * intNLanguages + intLanguage))
        cboDarkness.Items.Add(strTexts(intListfrmCriterionBase + 10 * intNLanguages + intLanguage))
        blnIsInitializing = False 'Force an update
        cboDarkness.Items.Add(strTexts(intListfrmCriterionBase + 11 * intNLanguages + intLanguage))
        cboMoonPart.Items.Clear()
        cboMoonPart.Items.Add(strTexts(intListfrmCriterionBase + 12 * intNLanguages + intLanguage))
        cboMoonPart.Items.Add(strTexts(intListfrmCriterionBase + 13 * intNLanguages + intLanguage))
        cboMoonPart.Items.Add(strTexts(intListfrmCriterionBase + 14 * intNLanguages + intLanguage))
        cboMoonBright.Items.Clear()
        cboMoonBright.Items.Add(strTexts(intListfrmCriterionBase + 15 * intNLanguages + intLanguage))
        cboMoonBright.Items.Add(strTexts(intListfrmCriterionBase + 16 * intNLanguages + intLanguage))
        cboMoonBright.Items.Add(strTexts(intListfrmCriterionBase + 17 * intNLanguages + intLanguage))
        cboMoonBright.Items.Add(strTexts(intListfrmCriterionBase + 18 * intNLanguages + intLanguage))
        cboMoonBright.Items.Add(strTexts(intListfrmCriterionBase + 19 * intNLanguages + intLanguage))
        cboMoonBright.Items.Add(strTexts(intListfrmCriterionBase + 20 * intNLanguages + intLanguage))
        cboMoonBright.Items.Add(strTexts(intListfrmCriterionBase + 21 * intNLanguages + intLanguage))
        cboFrom.Items.Clear()
        cboFrom.Items.Add(strTexts(intListfrmCriterionBase + 22 * intNLanguages + intLanguage))
        cboFrom.Items.Add(strTexts(intListfrmCriterionBase + 23 * intNLanguages + intLanguage))
        cboFrom.Items.Add(strTexts(intListfrmCriterionBase + 24 * intNLanguages + intLanguage))
        cboFrom.Items.Add(strTexts(intListfrmCriterionBase + 25 * intNLanguages + intLanguage))
        cboWhen.Items.Clear()
        cboWhen.Items.Add(strTexts(intListfrmCriterionBase + 26 * intNLanguages + intLanguage))
        cboWhen.Items.Add(strTexts(intListfrmCriterionBase + 27 * intNLanguages + intLanguage))
        cboWhen.Items.Add(strTexts(intListfrmCriterionBase + 28 * intNLanguages + intLanguage))
        cboWhen.Items.Add(strTexts(intListfrmCriterionBase + 29 * intNLanguages + intLanguage))
        Call MyRefresh()
    End Sub

    Private Sub frmCriterion_FormClosed(ByVal eventSender As System.Object, ByVal eventArgs As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        'Save and close if the closure is not cancelled
        If Not blnCancelled Then
            Call cmdOK_Click(cmdOK, New System.EventArgs())
        End If
    End Sub
    Private Sub MyRedraw()
        'Redraw the window after resizing
        Dim dblScaleSize As Double
        Dim sngScalePoint As System.Drawing.Point
        Dim sngScaleSize As System.Drawing.Size

        If Not Me.Visible Then Exit Sub
        If Me.ClientSize.Width < Me.ClientSize.Height * 1.194 And ReleaseVersion Or Me.ClientSize.Width < Me.ClientSize.Height * 1.058 And ReleaseVersion Then
            dblScaleSize = Me.ClientSize.Width / 258
        Else
            If ReleaseVersion Then
                dblScaleSize = Me.ClientSize.Height / 216
            Else
                dblScaleSize = Me.ClientSize.Height / 242
            End If
        End If
        If dblScaleSize < 0.1 Then Exit Sub 'Font too small to display
        Dim aFont As New System.Drawing.Font("Arial", CSng(8 * dblScaleSize))
        Call setsize(8, 9, 134, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblType.Location = sngScalePoint
        lblType.Size = sngScaleSize
        lblType.Font = aFont
        If blnRtl Then
            lblType.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblType.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(150, 8, 97, 22, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cboType.Location = sngScalePoint
        cboType.Size = sngScaleSize
        cboType.Font = aFont
        If blnRtl Then
            cboType.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cboType.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 39, 134, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblDark.Location = sngScalePoint
        lblDark.Size = sngScaleSize
        lblDark.Font = aFont
        If blnRtl Then
            lblDark.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblDark.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(150, 38, 97, 22, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cboDarkness.Location = sngScalePoint
        cboDarkness.Size = sngScaleSize
        cboDarkness.Font = aFont
        If blnRtl Then
            cboDarkness.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cboDarkness.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 69, 134, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblMoonPart.Location = sngScalePoint
        lblMoonPart.Size = sngScaleSize
        lblMoonPart.Font = aFont
        If blnRtl Then
            lblMoonPart.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblMoonPart.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(150, 68, 97, 22, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cboMoonPart.Location = sngScalePoint
        cboMoonPart.Size = sngScaleSize
        cboMoonPart.Font = aFont
        If blnRtl Then
            cboMoonPart.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cboMoonPart.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 99, 134, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblMoonBright.Location = sngScalePoint
        lblMoonBright.Size = sngScaleSize
        lblMoonBright.Font = aFont
        If blnRtl Then
            lblMoonBright.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblMoonBright.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(150, 98, 97, 22, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cboMoonBright.Location = sngScalePoint
        cboMoonBright.Size = sngScaleSize
        cboMoonBright.Font = aFont
        If blnRtl Then
            cboMoonBright.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cboMoonBright.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 129, 134, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblFrom.Location = sngScalePoint
        lblFrom.Size = sngScaleSize
        lblFrom.Font = aFont
        If blnRtl Then
            lblFrom.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblFrom.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(150, 128, 97, 22, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cboFrom.Location = sngScalePoint
        cboFrom.Size = sngScaleSize
        cboFrom.Font = aFont
        If blnRtl Then
            cboFrom.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cboFrom.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 160, 33, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblWhen.Location = sngScalePoint
        lblWhen.Size = sngScaleSize
        lblWhen.Font = aFont
        If blnRtl Then
            lblWhen.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblWhen.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(49, 159, 33, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtWhen.Location = sngScalePoint
        txtWhen.Size = sngScaleSize
        txtWhen.Font = aFont
        If blnRtl Then
            txtWhen.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtWhen.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(90, 158, 157, 22, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cboWhen.Location = sngScalePoint
        cboWhen.Size = sngScaleSize
        cboWhen.Font = aFont
        If blnRtl Then
            cboWhen.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cboWhen.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(36, 188, 65, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cmdDefault.Location = sngScalePoint
        cmdDefault.Size = sngScaleSize
        cmdDefault.Font = aFont
        If blnRtl Then
            cmdDefault.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cmdDefault.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(182, 188, 65, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cmdOK.Location = sngScalePoint
        cmdOK.Size = sngScaleSize
        cmdOK.Font = aFont
        If blnRtl Then
            cmdOK.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cmdOK.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(109, 188, 65, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cmdCancel.Location = sngScalePoint
        cmdCancel.Size = sngScaleSize
        cmdCancel.Font = aFont
        If blnRtl Then
            cmdCancel.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cmdCancel.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        If Not ReleaseVersion Then
            Call setsize(8, 216, 65, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
            cmdTest.Location = sngScalePoint
            cmdTest.Size = sngScaleSize
            cmdTest.Font = aFont
            If blnRtl Then
                cmdTest.RightToLeft = Windows.Forms.RightToLeft.Yes
            Else
                cmdTest.RightToLeft = Windows.Forms.RightToLeft.No
            End If
        End If
    End Sub

    Private Sub frmCriterion_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        Call MyRedraw()
    End Sub

    Private Sub frmCriterion_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.VisibleChanged
        Call MyRedraw()
    End Sub


    Private Sub cmdDefault_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDefault.Click
        'Set the default parameters
        'If the paramters are not already the defaults then confirm the command
        If cboType.SelectedIndex <> 2 Or _
            cboDarkness.SelectedIndex <> 2 Or _
            cboMoonPart.SelectedIndex <> 0 Or _
            cboMoonBright.SelectedIndex <> 3 Or _
            cboFrom.SelectedIndex <> 3 Or _
            txtWhen.Text <> "0.6" Or _
            cboWhen.SelectedIndex <> 2 Then 'If not already default then confirm change
            strMsgBox = strTexts(intStringBase + 6 * intNLanguages + intLanguage)
            MyMsgBox.ShowDialog()
            If MyMsgBox.DialogResult = System.Windows.Forms.DialogResult.OK Then
                'If confirmed then change to default parameters
                Call SetDefaults()
                Call MyRefresh()
            End If
        End If
    End Sub
End Class