Option Strict Off
Option Explicit On
Friend Class frmWeather
	Inherits System.Windows.Forms.Form
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Deal with weather related commands
	
    Dim blnCancelled As Boolean
    Dim blnIsInitializing As Boolean
	
	Friend Sub GetMeanWeather(ByRef udtWeatherNow As WeatherNow)
		'Get average weather for the current location
		Dim dblMinDistance As Double
		Dim dblDistance As Double
        'Dim dblHeightDif As Double
		Dim intCountry As Short
		Dim intCity As Short
		Dim intNeighborhood As Short
		Dim dblHeightDiff As Double
		
        Select Case udtWeather.udtBasis
            Case Main.WeatherBasis.fixedw 'Use the entered parameters
                udtWeatherNow.dblHumidity = udtWeather.udtClimate.udtHumidity.dblMean
                udtWeatherNow.dblPressure = udtWeather.udtClimate.udtPressure.dblMean
                udtWeatherNow.dblTemperature = udtWeather.udtClimate.udtTemperature.dblMean
            Case Main.WeatherBasis.manual 'Use the entered parameters
                udtWeatherNow.dblHumidity = udtWeather.udtClimate.udtHumidity.dblMean
                udtWeatherNow.dblPressure = udtWeather.udtClimate.udtPressure.dblMean
                udtWeatherNow.dblTemperature = udtWeather.udtClimate.udtTemperature.dblMean
            Case Main.WeatherBasis.GlobalW 'Based on global approximation
                udtWeatherNow.dblHumidity = 50 'Default value
                udtWeatherNow.dblPressure = 1010 + 10 * sinD(2 * udtPosition.dblLatitude) 'Good above 45S
                If udtPosition.dblLatitude < -45 Then 'Correct for low pressure in the Antarctic
                    udtWeatherNow.dblPressure = udtWeatherNow.dblPressure + (udtPosition.dblLatitude + 45) / 3
                End If
                udtWeatherNow.dblPressure = udtWeatherNow.dblPressure / System.Math.Exp(udtPosition.dblHeight / 7143) 'Adjust for height above sea level
                udtWeatherNow.dblTemperature = 27 - System.Math.Abs(udtPosition.dblLatitude) * 0.467 + dblHeightTempAdj(udtPosition.dblHeight, 0) 'Estimate temperature
            Case Main.WeatherBasis.LocalW
                dblMinDistance = 1000000 'Impossibly large
                'Find closest location
                For intCountry = 0 To udtLocationData.intNStatesCountries - 1
                    For intCity = 0 To udtLocationData.udtCountryState(intCountry).intNCities - 1
                        For intNeighborhood = 0 To udtLocationData.udtCountryState(intCountry).udtCity(intCity).intNNeighborhoods - 1
                            With udtLocationData.udtCountryState(intCountry).udtCity(intCity).udtNeighborhood(intNeighborhood).udtPosition
                                dblDistance = sinD(.dblLatitude) * sinD(udtPosition.dblLatitude) + cosD(.dblLatitude) * cosD(udtPosition.dblLatitude) * cosD(.dblLongitude - udtPosition.dblLongitude)
                                If dblDistance > 0.999995 Then
                                    dblDistance = 117.48 * System.Math.Sqrt(Square(udtPosition.dblLongitude - .dblLongitude) * cosD((.dblLatitude + udtPosition.dblLatitude) / 2) + Square(.dblLatitude - udtPosition.dblLatitude))
                                ElseIf dblDistance <= -1 Then
                                    dblDistance = 20015.19
                                Else
                                    dblDistance = 117.48 * acosD(dblDistance)
                                End If
                                dblHeightDiff = udtPosition.dblHeight - .dblHeight
                            End With
                            If dblDistance < dblMinDistance Then
                                dblMinDistance = dblDistance
                                'Set the weahter parameters adjusting for change in height above sea level
                                With udtLocationData.udtCountryState(intCountry).udtCity(intCity).udtNeighborhood(intNeighborhood).udtLocalClimate
                                    udtWeatherNow.dblHumidity = .udtHumidity.dblMean
                                    udtWeatherNow.dblPressure = .udtPressure.dblMean / System.Math.Exp(dblHeightDiff / 7143)
                                    udtWeatherNow.dblTemperature = .udtTemperature.dblMean + dblHeightTempAdj(udtPosition.dblHeight, udtLocationData.udtCountryState(intCountry).udtCity(intCity).udtNeighborhood(intNeighborhood).udtPosition.dblHeight)
                                End With
                            End If
                        Next
                    Next
                Next
            Case Else
                MsgBox(strTexts(intfrmWeatherBase + 8 * intNLanguages + intLanguage))
        End Select
	End Sub
    Friend Sub GetWeatherNow(ByVal dblFix As Double, ByRef udtWeatherNow As WeatherNow, ByRef blnValid As Boolean)
        'Get weather appropriate for the current location and time
        Dim dblMinDistance As Double
        Dim dblDistance As Double
        Dim dblHeightDif As Double
        Dim intCountry As Short
        Dim intCity As Short
        Dim intNeighborhood As Short
        Dim dblAnnual As Double
        Dim dblDaily As Double
        Dim cSun As New clsSun
        Dim dblHeightDiff As Double

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax + 2 Then 'allow leaway for calls near the limit
            Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Magnitude of dblFix too large: " & dblFix)
        End If
        If udtPosition.dblHeight < HeightMin Or udtPosition.dblHeight > HeightMax Then
            Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Invalid udtPosition.dblHeight: " & udtPosition.dblHeight)
        End If
        If udtPosition.dblLatitude < -90 Or udtPosition.dblLatitude > 90 Then
            Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Invalid udtPosition.dblLatitude: " & udtPosition.dblLatitude)
        End If
        If udtPosition.dblLongitude <= -180 Or udtPosition.dblLongitude > 180 Then
            Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Invalid udtPosition.dblLongitude: " & udtPosition.dblLongitude)
        End If
        'Annual parameter reaches maximum in late July and minimum in late January
        dblAnnual = cosD(cSun.MeanSun(dblFix, blnValid) - 128) / 2
        If Not blnValid Then Exit Sub
        'Daily paramter reaches maximum at 2:30pm mean solar time and minimum at 2:30am
        'Calculates hour angle
        dblDaily = cosD((dblFix - Int(dblFix)) * 360 + udtLocation.dblLongitude - 218) / 2
        Select Case udtWeather.udtBasis
            Case Main.WeatherBasis.fixedw
                With udtWeather.udtClimate
                    udtWeatherNow.dblHumidity = .udtHumidity.dblMean
                    udtWeatherNow.dblPressure = .udtPressure.dblMean
                    udtWeatherNow.dblTemperature = .udtTemperature.dblMean
                End With
            Case Main.WeatherBasis.manual
                With udtWeather.udtClimate
                    udtWeatherNow.dblHumidity = .udtHumidity.dblMean + dblAnnual * .udtHumidity.dblAnnualRange + dblDaily * .udtHumidity.dblDailyRange
                    udtWeatherNow.dblPressure = .udtPressure.dblMean + dblAnnual * .udtPressure.dblAnnualRange + dblDaily * .udtPressure.dblDailyRange
                    udtWeatherNow.dblTemperature = .udtTemperature.dblMean + dblAnnual * .udtTemperature.dblAnnualRange + dblDaily * .udtTemperature.dblDailyRange
                End With
            Case Main.WeatherBasis.GlobalW
                udtWeatherNow.dblHumidity = 50
                udtWeatherNow.dblPressure = 1010 + 10 * sinD(2 * udtPosition.dblLatitude)
                If udtPosition.dblLatitude < -45 Then
                    udtWeatherNow.dblPressure = udtWeatherNow.dblPressure + (udtPosition.dblLatitude + 45) / 3
                End If
                udtWeatherNow.dblPressure = udtWeatherNow.dblPressure / System.Math.Exp(udtPosition.dblHeight / 7143)
                udtWeatherNow.dblTemperature = 27 - System.Math.Abs(udtPosition.dblLatitude) * 0.467 + dblHeightTempAdj(udtPosition.dblHeight, 0) + 50 * sinD(udtLocation.dblLatitude) * dblAnnual + 2 * dblDaily
            Case Main.WeatherBasis.LocalW
                If udtLastWeatherPosition.dblHeight = udtPosition.dblHeight And udtLastWeatherPosition.dblLatitude = udtPosition.dblLatitude And udtLastWeatherPosition.dblLongitude = udtPosition.dblLongitude Then
                    With udtLocationData.udtCountryState(intLastWeatherCountry).udtCity(intLastWeatherCity).udtNeighborhood(intLastWeatherNeighborhood).udtLocalClimate
                        udtWeatherNow.dblHumidity = .udtHumidity.dblMean + .udtHumidity.dblAnnualRange * dblAnnual + .udtHumidity.dblDailyRange * dblDaily
                        udtWeatherNow.dblPressure = (.udtPressure.dblMean + .udtPressure.dblAnnualRange * dblAnnual + .udtPressure.dblDailyRange * dblDaily) / System.Math.Exp(dblHeightDiff / 7143)
                        udtWeatherNow.dblTemperature = .udtTemperature.dblMean + dblHeightTempAdj(udtPosition.dblHeight, udtPosition.dblHeight - dblHeightDiff) + .udtTemperature.dblAnnualRange * dblAnnual + .udtTemperature.dblDailyRange * dblDaily
                    End With
                Else
                    dblMinDistance = 1000000 'Impossibly large
                    For intCountry = 0 To udtLocationData.intNStatesCountries - 1
                        For intCity = 0 To udtLocationData.udtCountryState(intCountry).intNCities - 1
                            For intNeighborhood = 0 To udtLocationData.udtCountryState(intCountry).udtCity(intCity).intNNeighborhoods - 1
                                With udtLocationData.udtCountryState(intCountry).udtCity(intCity).udtNeighborhood(intNeighborhood).udtPosition
                                    dblDistance = sinD(.dblLatitude) * sinD(udtPosition.dblLatitude) + cosD(.dblLatitude) * cosD(udtPosition.dblLatitude) * cosD(.dblLongitude - udtPosition.dblLongitude)
                                    If dblDistance > 0.999995 Then
                                        dblDistance = Square(udtPosition.dblLongitude - .dblLongitude) * cosD((.dblLatitude + udtPosition.dblLatitude) / 2) + Square(.dblLatitude - udtPosition.dblLatitude)
                                        If dblDistance < 0 Then
                                            dblDistance = 0
                                        Else
                                            dblDistance = 117.48 * System.Math.Sqrt(dblDistance)
                                        End If
                                    ElseIf dblDistance <= -1 Then
                                        dblDistance = 20015.19
                                    Else
                                        dblDistance = 117.48 * acosD(dblDistance)
                                    End If
                                    dblHeightDiff = udtPosition.dblHeight - .dblHeight
                                End With
                                If dblDistance < dblMinDistance Then
                                    dblMinDistance = dblDistance
                                    With udtLocationData.udtCountryState(intCountry).udtCity(intCity).udtNeighborhood(intNeighborhood).udtLocalClimate
                                        udtWeatherNow.dblHumidity = .udtHumidity.dblMean + .udtHumidity.dblAnnualRange * dblAnnual + .udtHumidity.dblDailyRange * dblDaily
                                        udtWeatherNow.dblPressure = (.udtPressure.dblMean + .udtPressure.dblAnnualRange * dblAnnual + .udtPressure.dblDailyRange * dblDaily) / System.Math.Exp(dblHeightDiff / 7143)
                                        udtWeatherNow.dblTemperature = .udtTemperature.dblMean + dblHeightTempAdj(udtPosition.dblHeight, udtPosition.dblHeight - dblHeightDiff) + .udtTemperature.dblAnnualRange * dblAnnual + .udtTemperature.dblDailyRange * dblDaily
                                        udtLastWeatherPosition.dblHeight = udtPosition.dblHeight
                                        udtLastWeatherPosition.dblLatitude = udtPosition.dblLatitude
                                        udtLastWeatherPosition.dblLatitude = udtPosition.dblLongitude
                                        intLastWeatherCountry = intCountry
                                        intLastWeatherCity = intCity
                                        intLastWeatherNeighborhood = intNeighborhood
                                    End With
                                End If
                            Next
                        Next
                    Next
                End If
            Case Else
                Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Invalid weather basis: " & udtWeather.udtBasis)
        End Select
        If udtWeatherNow.dblHumidity < 0 Or udtWeatherNow.dblHumidity > 100 Then
            Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Invalid humidity: " & udtWeatherNow.dblHumidity)
        End If
        If udtWeatherNow.dblPressure < 0 Or udtWeatherNow.dblHumidity > 1200 Then
            Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Invalid pressure: " & udtWeatherNow.dblPressure)
        End If
        '    If udtWeatherNow.dblTemperature < -90 Then udtWeatherNow.dblTemperature = -90
        If udtWeatherNow.dblTemperature < -120 Or udtWeatherNow.dblTemperature > 70 Then
            Err.Raise(vbObjectError + 1, "frmWeather.GetWeatherNow", "Invalid temperature: " & udtWeatherNow.dblTemperature)
        End If


        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function WriteWeather(ByVal udtWeatherNow As WeatherNow, ByRef blnValid As Boolean) As String
        'Determine and print the current weather
        On Error GoTo EH
        blnValid = True
        WriteWeather = strTexts(intfrmWeatherBase + intNLanguages + intLanguage) & " "
        Select Case udtWeather.udtBasis
            Case Main.WeatherBasis.GlobalW
                WriteWeather = WriteWeather & strTexts(intListfrmWeatherBase + intLanguage) & " "
            Case Main.WeatherBasis.LocalW
                WriteWeather = WriteWeather & strTexts(intListfrmWeatherBase + intNLanguages + intLanguage) & " "
            Case Main.WeatherBasis.manual
                WriteWeather = WriteWeather & strTexts(intListfrmWeatherBase + 2 * intNLanguages + intLanguage) & " "
            Case Main.WeatherBasis.fixedw
                WriteWeather = WriteWeather & strTexts(intListfrmWeatherBase + 3 * intNLanguages + intLanguage) & " "
            Case Else
                Err.Raise(vbObjectError + 1, "frmWeather.WriteWeather", "Invalid udtWeather.udtBasis: " & udtWeather.udtBasis)
        End Select
        If udtWeatherNow.dblTemperature < TMin Or udtWeatherNow.dblTemperature > TMax Then
            Err.Raise(vbObjectError + 1, "frmWeather.WriteWeather", "Invalid udtWeatherNow.dblTemperature: " & udtWeatherNow.dblTemperature)
        End If
        If udtWeatherNow.dblPressure < PMin Or udtWeatherNow.dblPressure > PMax Then
            Err.Raise(vbObjectError + 1, "frmWeather.WriteWeather", "Invalid udtWeatherNow.dblPressure: " & udtWeatherNow.dblPressure)
        End If
        If udtWeatherNow.dblHumidity < HMin Or udtWeatherNow.dblHumidity > HMax Then
            Err.Raise(vbObjectError + 1, "frmWeather.WriteWeather", "Invalid udtWeatherNow.dblHumidity: " & udtWeatherNow.dblHumidity)
        End If
        WriteWeather = WriteWeather & strTexts(intfrmWeatherBase + 9 * intNLanguages + intLanguage) & " " & VB6.Format(udtWeatherNow.dblTemperature, "###.0") & " C, "
        WriteWeather = WriteWeather & strTexts(intfrmWeatherBase + 10 * intNLanguages + intLanguage) & " " & VB6.Format(udtWeatherNow.dblPressure, "####") & " mbar, "
        WriteWeather = WriteWeather & strTexts(intfrmWeatherBase + 11 * intNLanguages + intLanguage) & " " & VB6.Format(udtWeatherNow.dblHumidity, "###") & " %"
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Private Sub cboBasis_SelectedIndexChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cboBasis.SelectedIndexChanged
        'Update the form with change of basis
        If Not blnIsInitializing Then
            Call BasisChange()
        End If
    End Sub
    Private Sub BasisChange()
        'Update the form with change of basis
        Dim blnManual As Boolean
        Dim blnFixedW As Boolean

        blnManual = (cboBasis.SelectedIndex = Main.WeatherBasis.manual)
        blnFixedW = (cboBasis.SelectedIndex = Main.WeatherBasis.fixedw)
        txtTM.Enabled = blnManual Or blnFixedW
        txtTD.Enabled = blnManual
        txtTA.Enabled = blnManual
        txtPM.Enabled = blnManual Or blnFixedW
        txtPD.Enabled = blnManual
        txtPA.Enabled = blnManual
        txtHM.Enabled = blnManual Or blnFixedW
        txtHD.Enabled = blnManual
        txtHA.Enabled = blnManual
    End Sub
	
    Private Sub cmdCancel_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdCancel.Click
        'Deal with the cancel command
        If blnChanged() Then
            strMsgBox = strTexts(intStringBase + 5 * intNLanguages + intLanguage)
            MyMsgBox.ShowDialog()
            If MyMsgBox.DialogResult = System.Windows.Forms.DialogResult.OK Then 'Confirm cancel
                Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
                blnCancelled = True
                blnChangedInput = False
                Me.Close()
            End If
        Else 'If the form has not been changed, exit without confirming
            Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
            blnCancelled = True
            blnChangedInput = False
            Me.Close()
        End If
    End Sub
	
    Private Sub cmdOK_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdOK.Click
        'Set the paramters and close the form
        Dim blnError As Boolean
        Dim blnLocalError As Boolean
        Dim strError As String
        Dim intI As Short

        blnError = False
        strError = strTexts(intfrmWeatherBase + 12 * intNLanguages + intLanguage)
        blnLocalError = False
        If cboBasis.SelectedIndex = -1 Then
            strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 8 * intNLanguages + intLanguage)
            blnError = True
            MsgBox(strError)
        Else 'If cboBasis.SelectedIndex = Main.WeatherBasis.fixedw Then 
            If Not IsNumeric(txtTM.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 13 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf CDbl(txtTM.Text) < TSetMin Or CDbl(txtTM.Text) > TMax Then
                strError = strTexts(intfrmWeatherBase + 14 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If blnLocalError Then blnError = True
            blnLocalError = False
            If Not IsNumeric(txtPM.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 15 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf CDbl(txtPM.Text) < PMin Or CDbl(txtPM.Text) > PSetMax Then
                strError = strTexts(intfrmWeatherBase + 16 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If blnLocalError Then blnError = True
            blnLocalError = False
            If Not IsNumeric(txtHM.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 17 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf CDbl(txtHM.Text) < HMin Or CDbl(txtHM.Text) > HMax Then
                strError = strTexts(intfrmWeatherBase + 18 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If blnLocalError Then blnError = True
            'Load parameters
            If Not IsNumeric(txtTM.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 13 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf CDbl(txtTM.Text) < TSetMin Or CDbl(txtTM.Text) > TMax Then
                strError = strTexts(intfrmWeatherBase + 14 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not IsNumeric(txtTD.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 19 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf System.Math.Abs(CDbl(txtTD.Text)) > TMax - TMin Then
                strError = strTexts(intfrmWeatherBase + 20 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not IsNumeric(txtTA.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 21 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf System.Math.Abs(CDbl(txtTA.Text)) > TMax - TMin Then
                strError = strTexts(intfrmWeatherBase + 22 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not blnLocalError Then
                If CDbl(txtTM.Text) + System.Math.Abs(CDbl(txtTD.Text) + CDbl(txtTA.Text)) / 2 > TMax Or CDbl(txtTM.Text) - System.Math.Abs(CDbl(txtTD.Text) + CDbl(txtTA.Text)) / 2 < TSetMin Then
                    strError = strTexts(intfrmWeatherBase + 23 * intNLanguages + intLanguage)
                    blnLocalError = True
                End If
            End If
            If blnLocalError Then blnError = True
            blnLocalError = False
            If Not IsNumeric(txtPM.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 15 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf CDbl(txtPM.Text) < PMin Or CDbl(txtPM.Text) > PSetMax Then
                strError = strTexts(intfrmWeatherBase + 16 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not IsNumeric(txtPD.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 24 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf System.Math.Abs(CDbl(txtPD.Text)) > PSetMax - PMin Then
                strError = strTexts(intfrmWeatherBase + 25 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not IsNumeric(txtPA.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 26 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf System.Math.Abs(CDbl(txtPA.Text)) > PSetMax - PMin Then
                strError = strTexts(intfrmWeatherBase + 27 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not blnLocalError Then
                If CDbl(txtPM.Text) + System.Math.Abs(CDbl(txtPD.Text) + CDbl(txtPA.Text)) / 2 > PSetMax Or CDbl(txtPM.Text) - System.Math.Abs(CDbl(txtPD.Text) + CDbl(txtPA.Text)) / 2 < PMin Then
                    strError = strTexts(intfrmWeatherBase + 28 * intNLanguages + intLanguage)
                    blnLocalError = True
                End If
            End If
            If blnLocalError Then blnError = True
            blnLocalError = False
            If Not IsNumeric(txtHM.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 17 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf CDbl(txtHM.Text) < HMin Or CDbl(txtHM.Text) > HMax Then
                strError = strTexts(intfrmWeatherBase + 18 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not IsNumeric(txtHD.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 29 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf System.Math.Abs(CDbl(txtHD.Text)) > HMax - HMin Then
                strError = strTexts(intfrmWeatherBase + 30 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not IsNumeric(txtHA.Text) Then
                strError = strError & vbCrLf & strTexts(intfrmWeatherBase + 31 * intNLanguages + intLanguage)
                blnLocalError = True
            ElseIf System.Math.Abs(CDbl(txtHA.Text)) > HMax - HMin Then
                strError = strTexts(intfrmWeatherBase + 32 * intNLanguages + intLanguage)
                blnLocalError = True
            End If
            If Not blnLocalError Then
                If CDbl(txtHM.Text) + System.Math.Abs(CDbl(txtHD.Text) + CDbl(txtHA.Text)) / 2 > HMax Or CDbl(txtHM.Text) - System.Math.Abs(CDbl(txtHD.Text) + CDbl(txtHA.Text)) / 2 < HMin Then
                    strError = strTexts(intfrmWeatherBase + 33 * intNLanguages + intLanguage)
                    blnLocalError = True
                End If
            End If
            If blnLocalError Then blnError = True
            If blnError Then
                MsgBox(strError)
            Else
                If blnChanged() Then
                    Me.DialogResult = Windows.Forms.DialogResult.OK
                    With udtWeather.udtClimate
                        For intI = 0 To 6
                            udtVisibilitySave(intI).blnUpdated = False
                        Next
                        udtWeather.udtBasis = cboBasis.SelectedIndex
                        .udtHumidity.dblAnnualRange = CDbl(txtHA.Text)
                        .udtHumidity.dblDailyRange = CDbl(txtHD.Text)
                        .udtHumidity.dblMean = CDbl(txtHM.Text)
                        .udtPressure.dblAnnualRange = CDbl(txtPA.Text)
                        .udtPressure.dblDailyRange = CDbl(txtPD.Text)
                        .udtPressure.dblMean = CDbl(txtPM.Text)
                        .udtTemperature.dblAnnualRange = CDbl(txtTA.Text)
                        .udtTemperature.dblDailyRange = CDbl(txtTD.Text)
                        .udtTemperature.dblMean = CDbl(txtTM.Text)
                    End With
                Else
                    Me.DialogResult = Windows.Forms.DialogResult.Cancel
                End If
                blnCancelled = True
                Me.Close()
            End If
        End If
    End Sub
	
    Private Sub frmWeather_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        'Load the texts accoring to language and parameters
        blnIsInitializing = True
        Me.Text = strTexts(intfrmWeatherBase + intLanguage)
        lblBasis.Text = strTexts(intfrmWeatherBase + intNLanguages + intLanguage)
        lblMean.Text = strTexts(intfrmWeatherBase + 2 * intNLanguages + intLanguage)
        lblDaily.Text = strTexts(intfrmWeatherBase + 3 * intNLanguages + intLanguage)
        lblAnnual.Text = strTexts(intfrmWeatherBase + 4 * intNLanguages + intLanguage)
        lblTemperature.Text = strTexts(intfrmWeatherBase + 5 * intNLanguages + intLanguage)
        lblPressure.Text = strTexts(intfrmWeatherBase + 6 * intNLanguages + intLanguage)
        lblHumidity.Text = strTexts(intfrmWeatherBase + 7 * intNLanguages + intLanguage)
        cmdCancel.Text = strTexts(intStringBase + intNLanguages + intLanguage)
        cmdDefault.Text = strTexts(intStringBase + intLanguage)
        cmdOK.Text = strTexts(intStringBase + 2 * intNLanguages + intLanguage)
        cboBasis.Items.Clear()
        cboBasis.Items.Add(strTexts(intListfrmWeatherBase + intLanguage))
        cboBasis.Items.Add(strTexts(intListfrmWeatherBase + intNLanguages + intLanguage))
        cboBasis.Items.Add(strTexts(intListfrmWeatherBase + 2 * intNLanguages + intLanguage))
        cboBasis.Items.Add(strTexts(intListfrmWeatherBase + 3 * intNLanguages + intLanguage))
        cboBasis.Text = strTexts(intListfrmWeatherBase + intNLanguages + intLanguage)
        cboBasis.SelectedIndex = udtWeather.udtBasis
        Select Case udtWeather.udtBasis
            Case 0
                cboBasis.Text = strTexts(intListfrmWeatherBase + intLanguage) '"Global"
            Case 1
                cboBasis.Text = strTexts(intListfrmWeatherBase + intNLanguages + intLanguage) '"Local"
            Case 2
                cboBasis.Text = strTexts(intListfrmWeatherBase + 2 * intNLanguages + intLanguage) '"Manual"
            Case 3
                cboBasis.Text = strTexts(intListfrmWeatherBase + 3 * intNLanguages + intLanguage) '"Fixed"
        End Select
        cboBasis_SelectedIndexChanged(cboBasis, New System.EventArgs())
        With udtWeather.udtClimate
            txtTM.Text = CStr(.udtTemperature.dblMean)
            txtTD.Text = CStr(.udtTemperature.dblDailyRange)
            txtTA.Text = CStr(.udtTemperature.dblAnnualRange)
            txtPM.Text = CStr(.udtPressure.dblMean)
            txtPD.Text = CStr(.udtPressure.dblDailyRange)
            txtPA.Text = CStr(.udtPressure.dblAnnualRange)
            txtHM.Text = CStr(.udtHumidity.dblMean)
            txtHD.Text = CStr(.udtHumidity.dblDailyRange)
            txtHA.Text = CStr(.udtHumidity.dblAnnualRange)
        End With
        Call BasisChange()
        blnCancelled = False
        blnIsInitializing = False
    End Sub
	
    Private Sub frmWeather_FormClosed(ByVal eventSender As System.Object, ByVal eventArgs As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        'If not marked a cancelled then close
        If Not blnCancelled And Me.Visible Then Call cmdOK_Click(cmdOK, New System.EventArgs())
    End Sub
    Function dblHeightTempAdj(ByRef dblHeight As Double, ByRef dblRefHeight As Double) As Double
        'Adjust the temperature for height
        'Good up to 100 km above sea level
        If dblHeight < 10000 Then
            dblHeightTempAdj = -0.0065 * (dblHeight - dblRefHeight)
        ElseIf dblHeight < 20000 Then
            dblHeightTempAdj = -0.0065 * (10000 - dblRefHeight)
        ElseIf dblHeight < 45000 Then
            dblHeightTempAdj = -0.0065 * (10000 - dblRefHeight) + 0.002 * (dblHeight - 20000)
        ElseIf dblHeight < 50000 Then
            dblHeightTempAdj = -0.0065 * (10000 - dblRefHeight) + 0.002 * (45000 - 20000)
        ElseIf dblHeight < 90000 Then
            dblHeightTempAdj = -0.0065 * (10000 - dblRefHeight) + 0.002 * (45000 - 20000) - 0.00125 * (dblHeight - 50000)
        Else
            dblHeightTempAdj = -0.0065 * (10000 - dblRefHeight) + 0.002 * (45000 - 20000) - 0.00125 * (90000 - 50000) + 0.001 * (dblHeight - 90000)
        End If
    End Function
    Private Sub MyRedraw()
        'Redraw the window
        Dim dblScaleSize As Double
        Dim sngScalePoint As System.Drawing.Point
        Dim sngScaleSize As System.Drawing.Size

        If Not Me.Visible Then Exit Sub
        If Me.ClientSize.Width < Me.ClientSize.Height * 1.428 Then
            dblScaleSize = Me.ClientSize.Width / 267
        Else
            dblScaleSize = Me.ClientSize.Height / 187
        End If
        If dblScaleSize < 0.1 Then Exit Sub 'Font too small to display
        Dim aFont As New System.Drawing.Font("Arial", CSng(8 * dblScaleSize))
        Call setsize(8, 10, 33, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblBasis.Location = sngScalePoint
        lblBasis.Size = sngScaleSize
        lblBasis.Font = aFont
        If blnRtl Then
            lblBasis.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblBasis.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(49, 8, 89, 22, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cboBasis.Location = sngScalePoint
        cboBasis.Size = sngScaleSize
        cboBasis.Font = aFont
        If blnRtl Then
            cboBasis.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cboBasis.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(111, 38, 44, 29, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblMean.Location = sngScalePoint
        lblMean.Size = sngScaleSize
        lblMean.Font = aFont
        If blnRtl Then
            lblMean.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblMean.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(163, 38, 44, 29, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblDaily.Location = sngScalePoint
        lblDaily.Size = sngScaleSize
        lblDaily.Font = aFont
        If blnRtl Then
            lblDaily.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblDaily.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(215, 38, 44, 29, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblAnnual.Location = sngScalePoint
        lblAnnual.Size = sngScaleSize
        lblAnnual.Font = aFont
        If blnRtl Then
            lblAnnual.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblAnnual.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 76, 95, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblTemperature.Location = sngScalePoint
        lblTemperature.Size = sngScaleSize
        lblTemperature.Font = aFont
        If blnRtl Then
            lblTemperature.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblTemperature.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(111, 75, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtTM.Location = sngScalePoint
        txtTM.Size = sngScaleSize
        txtTM.Font = aFont
        If blnRtl Then
            txtTM.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtTM.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(163, 75, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtTD.Location = sngScalePoint
        txtTD.Size = sngScaleSize
        txtTD.Font = aFont
        If blnRtl Then
            txtTD.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtTD.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(215, 75, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtTA.Location = sngScalePoint
        txtTA.Size = sngScaleSize
        txtTA.Font = aFont
        If blnRtl Then
            txtTA.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtTA.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 104, 95, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblPressure.Location = sngScalePoint
        lblPressure.Size = sngScaleSize
        lblPressure.Font = aFont
        If blnRtl Then
            lblPressure.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblPressure.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(111, 103, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtPM.Location = sngScalePoint
        txtPM.Size = sngScaleSize
        txtPM.Font = aFont
        If blnRtl Then
            txtPM.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtPM.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(163, 103, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtPD.Location = sngScalePoint
        txtPD.Size = sngScaleSize
        txtPD.Font = aFont
        If blnRtl Then
            txtPD.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtPD.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(215, 103, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtPA.Location = sngScalePoint
        txtPA.Size = sngScaleSize
        txtPA.Font = aFont
        If blnRtl Then
            txtPA.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtPA.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(8, 132, 95, 17, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        lblHumidity.Location = sngScalePoint
        lblHumidity.Size = sngScaleSize
        lblHumidity.Font = aFont
        If blnRtl Then
            lblHumidity.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            lblHumidity.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(111, 131, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtHM.Location = sngScalePoint
        txtHM.Size = sngScaleSize
        txtHM.Font = aFont
        If blnRtl Then
            txtHM.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtHM.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(163, 131, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtHD.Location = sngScalePoint
        txtHD.Size = sngScaleSize
        txtHD.Font = aFont
        If blnRtl Then
            txtHD.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtHD.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(215, 131, 44, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        txtHA.Location = sngScalePoint
        txtHA.Size = sngScaleSize
        txtHA.Font = aFont
        If blnRtl Then
            txtHA.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            txtHA.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(72, 159, 57, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cmdDefault.Location = sngScalePoint
        cmdDefault.Size = sngScaleSize
        cmdDefault.Font = aFont
        If blnRtl Then
            cmdDefault.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cmdDefault.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(137, 159, 57, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cmdCancel.Location = sngScalePoint
        cmdCancel.Size = sngScaleSize
        cmdCancel.Font = aFont
        If blnRtl Then
            cmdCancel.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cmdCancel.RightToLeft = Windows.Forms.RightToLeft.No
        End If
        Call setsize(202, 159, 57, 20, dblScaleSize, Me.ClientSize.Width, sngScalePoint, sngScaleSize)
        cmdOK.Location = sngScalePoint
        cmdOK.Size = sngScaleSize
        cmdOK.Font = aFont
        If blnRtl Then
            cmdOK.RightToLeft = Windows.Forms.RightToLeft.Yes
        Else
            cmdOK.RightToLeft = Windows.Forms.RightToLeft.No
        End If
    End Sub

    Private Sub frmWeather_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        Call MyRedraw()
    End Sub

    Private Sub frmWeather_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.VisibleChanged
        Call MyRedraw()
    End Sub

    Private Sub cmdDefault_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmdDefault.Click
        'Deal with the default command
        'Confirm the change if not already default
        If Main.WeatherBasis.LocalW <> cboBasis.SelectedIndex Or "16" <> txtTM.Text Or "923.4" <> txtPM.Text Or "68.5" <> txtHM.Text Or _
                           "8.9" <> txtTD.Text Or "0" <> txtPD.Text Or "-18" <> txtHD.Text Or _
                           "8.6" <> txtTA.Text Or "-5.9" <> txtPA.Text Or "20.1" <> txtHA.Text Then
            strMsgBox = strTexts(intStringBase + 6 * intNLanguages + intLanguage)
            MyMsgBox.ShowDialog()
            If MyMsgBox.DialogResult = System.Windows.Forms.DialogResult.OK Then 'confirm before reverting to default values
                cboBasis.SelectedIndex = Main.WeatherBasis.LocalW
                txtTM.Text = "16"
                txtPM.Text = "923.4"
                txtHM.Text = "68.5"
                txtTD.Text = "8.9"
                txtPD.Text = "0"
                txtHD.Text = "-18"
                txtTA.Text = "8.6"
                txtPA.Text = "-5.9"
                txtHA.Text = "20.1"
            End If
        End If
    End Sub
    Private Function blnChanged() As Boolean
        'Determine if the parameters have been changed
        With udtWeather.udtClimate
            blnChanged = (udtWeather.udtBasis <> cboBasis.SelectedIndex Or _
            .udtTemperature.dblMean <> txtTM.Text Or .udtPressure.dblMean <> txtPM.Text Or .udtHumidity.dblMean <> txtHM.Text Or _
            .udtTemperature.dblDailyRange <> txtTD.Text Or .udtPressure.dblDailyRange <> txtPD.Text Or .udtHumidity.dblDailyRange <> txtHD.Text Or _
            .udtTemperature.dblAnnualRange <> txtTA.Text Or .udtPressure.dblAnnualRange <> txtPA.Text Or .udtHumidity.dblAnnualRange <> txtHA.Text)
        End With
    End Function
End Class